from __future__ import absolute_import


import unittest
import time
from testutils import ADMIN_CLIENT, suppress_urllib3_warning
from library.user import User


class TestUser(unittest.TestCase):


    @suppress_urllib3_warning
    def setUp(self):
        self.user = User()


    def testUser(self):
        """
        Test case:
            User CRUD
        Test step and expected result:
            1. Create a new user(UA);
            2. List all users, there should be one user(UA);
            3. Get current user by user(UA), it should be user(UA;
            4. Search user(UA) by name, it should be user(UA);
            5. Get user profile by user(UA), it should be user(UA);
            6. Update user profile by user(UA);
            7. Update user to admin;
            8. Update user password by user(UA);
            9. Update user password by admin;
            10. Get current user permissions by user(UA);
            11. Delete user(UA);
        """
        url = ADMIN_CLIENT["endpoint"]
        user_password = "Aa123456"

        # 1. Create a new user(UA);
        user_id, user_name = self.user.create_user(user_password=user_password, **ADMIN_CLIENT)
        timestamp = user_name.split("-")[1]
        USER_CLIENT=dict(endpoint=url, username=user_name, password=user_password)

        # 2. List all users, there should be one user(UA);
        users = self.user.get_users(**ADMIN_CLIENT)
        self.assertIsNotNone(users)

        # 3. Get current user by user(UA), it should be user(UA;
        current_user = self.user.get_user_current(**USER_CLIENT)
        self.check_user(current_user, user_name, user_id, timestamp)

        # 4. Search user(UA) by name, it should be user(UA);
        users = self.user.search_user_by_username(user_name, **USER_CLIENT)
        user = users[0]
        self.assertEqual(len(users), 1)
        self.assertEqual(user.username, user_name)
        self.assertEqual(user.user_id, user_id)

        # 5. Get user profile by user(UA), it should be user(UA);
        user = self.user.get_user_by_id(user_id, **USER_CLIENT)
        self.check_user(user, user_name, user_id, timestamp)

        # 6. Update user profile by user(UA);
        timestamp = int(round(time.time() * 1000))
        comment = "For testing"
        self.user.update_user_profile(user_id, email="realname-{}@harbortest.com".format(timestamp), realname="realname-{}".format(timestamp), comment=comment, **USER_CLIENT)
        user = self.user.get_user_by_id(user_id, **USER_CLIENT)
        self.check_user(user, user_name, user_id, timestamp, comment)

        # 7. Update user to admin;
        self.user.update_user_role_as_sysadmin(user_id, True, **ADMIN_CLIENT)
        user = self.user.get_user_by_id(user_id, **USER_CLIENT)
        self.check_user(user, user_name, user_id, timestamp, comment, True)

        # 8. Update user password by user(UA);
        new_password = "Aa1234567-New"
        self.user.update_user_pwd(user_id, new_password=new_password, old_password=user_password, **USER_CLIENT)
        self.user.search_user_by_username(user_name, expect_status_code=401, expect_response_body="unauthorized", **USER_CLIENT)
        USER_CLIENT["password"] = new_password

        # 9. Update user password by admin;
        new_password = "Aa1234567-New-Edit"
        self.user.update_user_pwd(user_id, new_password=new_password, old_password=USER_CLIENT["password"], **ADMIN_CLIENT)
        self.user.search_user_by_username(user_name, expect_status_code=401, expect_response_body="unauthorized", **USER_CLIENT)
        USER_CLIENT["password"] = new_password

        # 10. Get current user permissions by user(UA);
        permissions = self.user.get_current_user_permissions(scope="/project/1/repository", relative=True, **USER_CLIENT)
        self.assertTrue(len(permissions) > 0)

        # 11. Delete user(UA);
        self.user.delete_user(user_id, **ADMIN_CLIENT)
        self.user.get_user_by_id(user_id, expect_status_code=404, expect_response_body="user {} not found".format(user_id), **ADMIN_CLIENT)


    def check_user(self, user, user_name, user_id, timestamp, comment=None, sysadmin_flag=False):
        self.assertEqual(user.username, user_name)
        self.assertEqual(user.user_id, user_id)
        self.assertEqual(user.email, "realname-{}@harbortest.com".format(timestamp))
        self.assertEqual(user.comment, comment)
        self.assertEqual(user.realname, "realname-{}".format(timestamp))
        self.assertEqual(user.sysadmin_flag, sysadmin_flag)


if __name__ == '__main__':
    unittest.main()




from __future__ import absolute_import
import unittest

from testutils import ADMIN_CLIENT, suppress_urllib3_warning
from testutils import TEARDOWN
from library.project import Project
from library.user import User
from library.configurations import Configurations

class TestProjects(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.conf= Configurations()
        self.project= Project()
        self.user= User()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        #1. Delete project(PA);
        self.project.delete_project(TestProjects.project_edit_project_creation_id, **TestProjects.USER_edit_project_creation_CLIENT)

        #2. Delete user(UA);
        self.user.delete_user(TestProjects.user_edit_project_creation_id, **ADMIN_CLIENT)

    def testEditProjectCreation(self):
        """
        Test case:
            Edit Project Creation
        Test step and expected result:
            1. Create a new user(UA);
            2. Set project creation to "admin only";
            3. Create a new project(PA) by user(UA), and fail to create a new project;
            4. Set project creation to "everyone";
            5. Create a new project(PA) by user(UA), success to create a project.
        Tear down:
            1. Delete project(PA);
            2. Delete user(UA);
        """
        url = ADMIN_CLIENT["endpoint"]
        user_edit_project_creation_password = "Aa123456"

        #1. Create a new user(UA);
        TestProjects.user_edit_project_creation_id, user_edit_project_creation_name = self.user.create_user(user_password = user_edit_project_creation_password, **ADMIN_CLIENT)

        TestProjects.USER_edit_project_creation_CLIENT=dict(endpoint = url, username = user_edit_project_creation_name, password = user_edit_project_creation_password)

        #2. Set project creation to "admin only";
        self.conf.set_configurations_of_project_creation_restriction("adminonly", **ADMIN_CLIENT)

        #3. Create a new project(PA) by user(UA), and fail to create a new project;
        self.project.create_project(metadata = {"public": "false"}, expect_status_code = 403,
            expect_response_body = "{\"errors\":[{\"code\":\"FORBIDDEN\",\"message\":\"Only system admin can create project\"}]}", **TestProjects.USER_edit_project_creation_CLIENT)

        #4. Set project creation to "everyone";
        self.conf.set_configurations_of_project_creation_restriction("everyone", **ADMIN_CLIENT)

        #5. Create a new project(PA) by user(UA), success to create a project.
        TestProjects.project_edit_project_creation_id, _ = self.project.create_project(metadata = {"public": "false"}, **TestProjects.USER_edit_project_creation_CLIENT)


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import base
import v2_swagger_client

class Registry(base.Base, object):
    def __init__(self):
        super(Registry,self).__init__(api_type = "registry")

    def create_registry(self, url, registry_type= "harbor", description="", credentialType = "basic",
            access_key = "admin", access_secret = "Harbor12345", name=base._random_name("registry"),
            insecure=True, expect_status_code = 201, **kwargs):

        client = self._get_client(**kwargs)
        registryCredential = v2_swagger_client.RegistryCredential(type=credentialType, access_key=access_key, access_secret=access_secret)
        registry = v2_swagger_client.Registry(name=name, url=url,
                                           description= description, type=registry_type,
                                           insecure=insecure, credential=registryCredential)
        _, status_code, header = client.create_registry_with_http_info(registry)
        base._assert_status_code(expect_status_code, status_code)
        return base._get_id_from_header(header), _

    def get_registry_id_by_endpoint(self, endpoint, **kwargs):
        client = self._get_client(**kwargs)
        registries = client.list_registries()
        for registry in registries or []:
            if registry.endpoint == endpoint:
                return registry.id
        raise Exception("registry %s not found" % endpoint)

    def delete_registry(self, registry_id, expect_status_code = 200, **kwargs):
        client = self._get_client(**kwargs)
        _, status_code, _  = client.delete_registry_with_http_info(registry_id)
        base._assert_status_code(expect_status_code, status_code)


#!/usr/bin/python

import argparse
import json
import requests
import sys

parser = argparse.ArgumentParser()
parser.add_argument("-H", "--host", help="The Harbor server need to config")
parser.add_argument("-u", "--user", default="admin", help="The Harbor username")
parser.add_argument("-p", "--password", default="Harbor12345", help="The Harbor password")
parser.add_argument("-c", "--config", nargs='+', help="The configure settings <key>=<value>, it can take more than one configures")
args = parser.parse_args()
reqJson = {}
for item in args.config :
    configs = item.split("=", 1)
    key = configs[0].strip()
    value = configs[1].strip()
    if value.lower() in ['true', 'yes', '1'] :
        reqJson[key] = True
    elif value.lower() in ['false', 'no', '0'] :
        reqJson[key] = False
    elif value.isdigit() :
        reqJson[key] = int(value)
    else:
        reqJson[key] = value

# Sample Basic Auth Url with login values as username and password
url = "https://"+args.host+"/api/v2.0/configurations"
user = args.user
passwd = args.password

# Make a request to the endpoint using the correct auth values
auth_values = (user, passwd)
session = requests.Session()
session.verify = False
data = json.dumps(reqJson)
headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
response = session.put(url, auth=auth_values, data=data, headers=headers)

# Convert JSON to dict and print
if response.status_code == 200 :
    print("Configure setting success")
    print("values:"+data)
    sys.exit(0)
else:
    print("Failed with http return code:"+ str(response.status_code))
    sys.exit(1)



from __future__ import absolute_import
import unittest

from testutils import harbor_server, suppress_urllib3_warning
from testutils import admin_user
from testutils import admin_pwd
from testutils import TEARDOWN
from testutils import ADMIN_CLIENT
from library.system import System
from library.project import Project
from library.user import User
from library.repository import Repository
from library.repository import push_self_build_image_to_project
from library.artifact import Artifact
from library.docker_api import list_image_tags
from library.docker_api import list_repositories
import os
import library.base
import json

class TestProjects(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.system = System()
        self.project= Project()
        self.user= User()
        self.artifact = Artifact()
        self.repo = Repository()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        #1. Delete Alice's repository and Luca's repository;
        self.repo.delete_repository(TestProjects.project_Alice_name, TestProjects.repo_a.split('/')[1], **ADMIN_CLIENT)
        self.repo.delete_repository(TestProjects.project_Alice_name, TestProjects.repo_b.split('/')[1], **ADMIN_CLIENT)
        self.repo.delete_repository(TestProjects.project_Alice_name, TestProjects.repo_c.split('/')[1], **ADMIN_CLIENT)

        #2. Delete Alice's project and Luca's project;
        self.project.delete_project(TestProjects.project_Alice_id, **ADMIN_CLIENT)

        #3. Delete user Alice and Luca.
        self.user.delete_user(TestProjects.user_Alice_id, **ADMIN_CLIENT)

    def testRegistryAPI(self):
        """
        Test case:
            Catalog API to list all repositories by system admin
        Test step and expected result:G
            1. Create user Alice;
            2. Create 1 new private projects project_Alice;
            3. Push 3 images to project_Alice and Add 3 tags to the 3rd image
            4. Call the image_list_tag API
            5. Call the catalog API using admin account without pagination, can get all 3 images
            5.1 Call the catalog API using admin account with pagination n=1, page=2, twice to get all 3 images.
            5.2 Call the catalog API using Alice account, no repos should be found.
        Tear down:
            1. Delete Alice's repository;
            2. Delete Alice's project;
            3. Delete user Alice.
        """
        url = ADMIN_CLIENT["endpoint"]
        user_common_password = "Aa123456"
        #1. Create user Alice and Luca;
        TestProjects.user_Alice_id, user_Alice_name = self.user.create_user(user_password = user_common_password, **ADMIN_CLIENT)

        USER_ALICE_CLIENT=dict(endpoint = url, username = user_Alice_name, password = user_common_password)

        #2. Create 2 new private projects project_Alice and project_Luca;
        TestProjects.project_Alice_id, TestProjects.project_Alice_name = self.project.create_project(metadata = {"public": "false"}, **USER_ALICE_CLIENT)

        #3. Push 3 images to project_Alice and Add 3 tags to the 3rd image.

        src_tag = "latest"
        image_a = "image_a"
        TestProjects.repo_a, tag_a = push_self_build_image_to_project(TestProjects.project_Alice_name, harbor_server, user_Alice_name, user_common_password, image_a, src_tag)
        image_b = "image_b"
        TestProjects.repo_b, tag_b = push_self_build_image_to_project(TestProjects.project_Alice_name, harbor_server, user_Alice_name, user_common_password, image_b, src_tag)
        image_c = "image_c"
        TestProjects.repo_c, tag_c = push_self_build_image_to_project(TestProjects.project_Alice_name, harbor_server, user_Alice_name, user_common_password, image_c, src_tag)
        create_tags = ["1.0","2.0","3.0"]
        for tag in create_tags:
            self.artifact.create_tag(TestProjects.project_Alice_name, image_c, tag_c, tag, **USER_ALICE_CLIENT)
        #4. Call the image_list_tags API
        tags = list_image_tags(harbor_server,TestProjects.repo_c,user_Alice_name,user_common_password)
        for tag in create_tags:
            self.assertTrue(tags.count(tag)>0, "Expect tag: %s is not listed"%tag)
        page_tags = list_image_tags(harbor_server,TestProjects.repo_c,user_Alice_name,user_common_password,len(tags)/2+1)
        page_tags += list_image_tags(harbor_server,TestProjects.repo_c,user_Alice_name,user_common_password,len(tags)/2+1,tags[int(len(tags)/2)])
        for tag in create_tags:
            self.assertTrue(page_tags.count(tag)>0, "Expect tag: %s is not listed by the pagination query"%tag)
        #5. Call the catalog API;
        repos = list_repositories(harbor_server,admin_user,admin_pwd)
        self.assertTrue(repos.count(TestProjects.repo_a)>0 and repos.count(TestProjects.repo_b)>0 and repos.count(TestProjects.repo_c)>0, "Expected repo not found")
        for repo in [TestProjects.repo_a,TestProjects.repo_b,TestProjects.repo_c]:
            self.assertTrue(repos.count(repo)>0,"Expected repo: %s is not listed"%repo)
        page_repos = list_repositories(harbor_server,admin_user,admin_pwd,len(repos)/2+1)
        page_repos += list_repositories(harbor_server,admin_user,admin_pwd,len(repos)/2+1,repos[int(len(repos)/2)])
        for repo in [TestProjects.repo_a,TestProjects.repo_b,TestProjects.repo_c]:
            self.assertTrue(page_repos.count(repo)>0,"Expected repo: %s is not listed by the pagination query"%repo)

        null_repos = list_repositories(harbor_server,user_Alice_name,user_common_password)
        self.assertEqual(null_repos, "")

if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import
import unittest
import sys

from testutils import harbor_server, suppress_urllib3_warning
from testutils import TEARDOWN
from testutils import ADMIN_CLIENT, BASE_IMAGE, BASE_IMAGE_ABS_PATH_NAME
from library.project import Project
from library.user import User
from library.repository import Repository
from library.repository import push_self_build_image_to_project
from library.artifact import Artifact
from library.scan import Scan

class TestScan(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.project= Project()
        self.user= User()
        self.artifact = Artifact()
        self.repo = Repository()
        self.scan = Scan()

        self.url = ADMIN_CLIENT["endpoint"]
        self.user_password = "Aa123456"
        self.project_id, self.project_name, self.user_id, self.user_name, self.repo_name1 = [None] * 5
        self.user_id, self.user_name = self.user.create_user(user_password = self.user_password, **ADMIN_CLIENT)
        self.USER_CLIENT = dict(with_immutable_status = True, endpoint = self.url, username = self.user_name, password = self.user_password, with_scan_overview = True)


        #2. Create a new private project(PA) by user(UA);
        self.project_id, self.project_name = self.project.create_project(metadata = {"public": "false"}, **ADMIN_CLIENT)

        #3. Add user(UA) as a member of project(PA) with project-admin role;
        self.project.add_project_members(self.project_id, user_id = self.user_id, **ADMIN_CLIENT)

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def do_tearDown(self):
        #1. Delete repository(RA) by user(UA);
        self.repo.delete_repository(self.project_name, self.repo_name1.split('/')[1], **self.USER_CLIENT)

        #2. Delete project(PA);
        self.project.delete_project(self.project_id, **self.USER_CLIENT)

        #3. Delete user(UA);
        self.user.delete_user(self.user_id, **ADMIN_CLIENT)

    def testScanImageArtifact(self):
        """
        Test case:
            Scan An Image Artifact
        Test step and expected result:
            1. Create a new user(UA);
            2. Create a new private project(PA) by user(UA);
            3. Add user(UA) as a member of project(PA) with project-admin role;
            4. Get private project of user(UA), user(UA) can see only one private project which is project(PA);
            5. Create a new repository(RA) and tag(TA) in project(PA) by user(UA);
            6. Send scan image command and get tag(TA) information to check scan result, it should be finished;
            7. Send scan another image command and get tag(TA) information to check scan result, it should be finished.
        Tear down:
            1. Delete repository(RA) by user(UA);
            2. Delete project(PA);
            3. Delete user(UA);
        """

        #4. Get private project of user(UA), user(UA) can see only one private project which is project(PA);
        self.project.projects_should_exist(dict(public=False), expected_count = 1,
            expected_project_id = self.project_id, **self.USER_CLIENT)

        #Note: Please make sure that this Image has never been pulled before by any other cases,
        #      so it is a not-scanned image right after repository creation.
        image = "docker"
        src_tag = "1.13"
        #5. Create a new repository(RA) and tag(TA) in project(PA) by user(UA);
        self.repo_name1, tag = push_self_build_image_to_project(self.project_name, harbor_server, self.user_name, self.user_password, image, src_tag)

        #6. Send scan image command and get tag(TA) information to check scan result, it should be finished;
        self.scan.scan_artifact(self.project_name, self.repo_name1.split('/')[1], tag, **self.USER_CLIENT)
        self.artifact.check_image_scan_result(self.project_name, image, tag, **self.USER_CLIENT)

        self.do_tearDown()

    def testScanSignedImage(self):
        """
        Test case:
            Scan A Signed Image
        Test step and expected result:
            1. Create a new user(UA);
            2. Create a new private project(PA) by user(UA);
            3. Add user(UA) as a member of project(PA) with project-admin role;
            4. Get private project of user(UA), user(UA) can see only one private project which is project(PA);
            5. Create a new repository(RA) and tag(TA) in project(PA) by user(UA);
            6. Send scan image command and get tag(TA) information to check scan result, it should be finished;
            7. Send scan another image command and get tag(TA) information to check scan result, it should be finished.
        Tear down:
            1. Delete repository(RA) by user(UA);
            2. Delete project(PA);
            3. Delete user(UA);
        """

        #Note: Please make sure that this Image has never been pulled before by any other cases,
        #      so it is a not-scanned image right after repository creation.
        #Note:busybox is pulled in setup phase, and setup is an essential phase before scripts execution.
        image = BASE_IMAGE['name']
        tag = BASE_IMAGE['tag']
        #5. Create a new repository(RA) and tag(TA) in project(PA) by user(UA);
        push_self_build_image_to_project(self.project_name, harbor_server, self.user_name, self.user_password, image, tag)

        #6. Send scan image command and get tag(TA) information to check scan result, it should be finished;
        self.scan.scan_artifact(self.project_name, image, tag, **self.USER_CLIENT)
        self.artifact.check_image_scan_result(self.project_name, image, tag, **self.USER_CLIENT)

if __name__ == '__main__':
    suite = unittest.TestSuite(unittest.makeSuite(TestScan))
    result = unittest.TextTestRunner(sys.stdout, verbosity=2, failfast=True).run(suite)
    if not result.wasSuccessful():
        raise Exception(r"Scan test failed: {}".format(result))



# -*- coding: utf-8 -*-

from __future__ import absolute_import
import unittest

from testutils import harbor_server, ADMIN_CLIENT, suppress_urllib3_warning
from library import podman
from library.project import Project
from library.user import User
from library.artifact import Artifact
from library.repository import push_self_build_image_to_project

class TestPodmanPullPush(unittest.TestCase):

    @suppress_urllib3_warning
    def setUp(self):
        self.project= Project()
        self.user= User()
        self.artifact = Artifact()
        self.image = "image_test"
        self.tag = "v1"
        self.source_image = "ghcr.io/goharbor/harbor-core"
        self.source_tag = "v2.8.2"

    def testPodman(self):
        """
        Test case:
            Podman pull and push
        Test step and expected result:
            1. Create a new user;
            2. Create a new project by user;
            3. Push a new image in project by user;
            4. Podman login harbor;
            5. Podman pull image from project(PA) by user;
            6. Podman pull soure image;
            7. Podman push soure image to project by user;
            8. Verify the image;
            9. Podman logout harbor;
        """
        url = ADMIN_CLIENT["endpoint"]
        user_password = "Aa123456"

        # 1. Create user(UA)
        _, user_name = self.user.create_user(user_password = user_password, **ADMIN_CLIENT)
        user_client = dict(endpoint = url, username = user_name, password = user_password, with_accessory = True)

        # 2. Create private project(PA) by user(UA)
        _, project_name = self.project.create_project(metadata = {"public": "false"}, **user_client)

        # 3. Push a new image(IA) in project(PA) by user(UA)
        push_self_build_image_to_project(project_name, harbor_server, user_name, user_password, self.image, self.tag)

        # 4. Podman login harbor
        podman.login(harbor_server, user_name, user_password)

        # 5. Podman pull image from project(PA) by user
        podman.pull("{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag))

        # 6. Podman pull soure image
        podman.pull("{}:{}".format(self.source_image, self.source_tag))

        # 7. Podman push soure image to project by user
        podman.push("{}:{}".format(self.source_image, self.source_tag), "{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag))

        # 8. Verify the image
        image_info = self.artifact.get_reference_info(project_name, self.image, self.tag, **user_client)
        self.assertIsNotNone(image_info)
        self.assertIsNotNone(image_info.digest)
        self.assertEqual(len(image_info.tags), 1)
        self.assertEqual(image_info.tags[0].name, self.tag)

        # 9. Podman logout harbor
        podman.logout(harbor_server)

if __name__ == '__main__':
    unittest.main()



import os
import sys
import click
import pathlib
import logging
from subprocess import Popen, PIPE, STDOUT, CalledProcessError

from utils.cert import openssl_installed
from utils.misc import get_realpath

gen_tls_script = pathlib.Path(__file__).parent.parent.joinpath('scripts/gencert.sh').absolute()

@click.command()
@click.option('-p', '--path', required=True, type=str,help='the path to store generated cert files')
@click.option('-d', '--days', default='365', type=str, help='the expired time for cert')
def gencert(path, days):
    """
    gencert command will generate cert files for internal TLS
    """
    path = get_realpath(path)
    click.echo('Check openssl ...')
    if not openssl_installed():
        raise(Exception('openssl not installed'))

    click.echo("start generate internal tls certs")
    if not os.path.exists(path):
        click.echo('path {} not exist, create it...'.format(path))
        os.makedirs(path, exist_ok=True)
    with Popen([gen_tls_script, days], stdout=PIPE, stderr=STDOUT, cwd=path) as p:
        for line in p.stdout:
            click.echo(line, nl=False)
    if p.returncode != 0:
        raise CalledProcessError(p.returncode, p.args)



import os
import sys
import warnings
from functools import wraps

sys.path.insert(0, os.environ.get("SWAGGER_CLIENT_PATH", ''))
path=os.getcwd() + "/library"
sys.path.insert(0, path)

path=os.getcwd() + "/tests/apitests/python/library"
sys.path.insert(0, path)
path=os.getcwd() + "/tests/apitests/python/"
sys.path.insert(0, path)
print(sys.path)

files_directory = os.getcwd() + "/tests/files/"

import importlib

admin_user = "admin"
admin_pwd = "Harbor12345"

harbor_server = os.environ.get("HARBOR_HOST", '')
harbor_url = "https://{}".format(harbor_server)
#CLIENT=dict(endpoint="https://"+harbor_server+"/api")
ADMIN_CLIENT=dict(endpoint = os.environ.get("HARBOR_HOST_SCHEMA", "https")+ "://"+harbor_server+"/api/v2.0", username = admin_user, password =  admin_pwd)
CHART_API_CLIENT=dict(endpoint = os.environ.get("HARBOR_HOST_SCHEMA", "https")+ "://"+harbor_server+"/api", username = admin_user, password =  admin_pwd)
USER_ROLE=dict(admin=0,normal=1)
TEARDOWN = os.environ.get('TEARDOWN', 'true').lower() in ('true', 'yes')
DOCKER_USER = os.environ.get('DOCKER_USER', '')
DOCKER_PWD = os.environ.get('DOCKER_PWD', '')
METRIC_URL = os.environ.get('METRIC_URL', 'http://'+harbor_server+':9090')
SYSLOG_ENDPOINT = os.environ.get('SYSLOG_ENDPOINT', '')
ES_ENDPOINT = os.environ.get('ES_ENDPOINT', '')
BASE_IMAGE = dict(name='busybox', tag='latest')
BASE_IMAGE_ABS_PATH_NAME = '/' + BASE_IMAGE['name'] + '.tar'

def swagger_module():
        module = importlib.import_module("v2_swagger_client")
        return module
 
def GetRepositoryApi(username, password, harbor_server= os.environ.get("HARBOR_HOST", '')):
    v2_swagger_client = swagger_module()
    cfg = v2_swagger_client.Configuration()
    cfg.host = "https://"+harbor_server+"/api/v2.0"
    cfg.username = username
    cfg.password = password
    cfg.verify_ssl = False
    cfg.debug = True
    api_client = v2_swagger_client.ApiClient(cfg)
    api_instance = v2_swagger_client.RepositoryApi(api_client)
    return api_instance

def GetUserGroupApi(username, password, harbor_server= os.environ.get("HARBOR_HOST", '')):
    v2_swagger_client = swagger_module()
    cfg = v2_swagger_client.Configuration()
    cfg.host = "https://"+harbor_server+"/api/v2.0"
    cfg.username = username
    cfg.password = password
    cfg.verify_ssl = False
    cfg.debug = True
    api_client = v2_swagger_client.ApiClient(cfg)
    api_instance = v2_swagger_client.UsergroupApi(api_client)
    return api_instance

class TestResult(object):
    def __init__(self):
        self.num_errors = 0
        self.error_message = []
    def add_test_result(self, error_message):
        self.num_errors = self.num_errors + 1
        self.error_message.append(error_message)
    def get_final_result(self):
        if self.num_errors > 0:
            for each_err_msg in self.error_message:
                print("Error message:", each_err_msg)
            raise Exception(r"Test case failed with {} errors.".format(self.num_errors))

def suppress_urllib3_warning(func):
    @wraps(func)
    def inner_func(*args):
        warnings.filterwarnings(action="ignore",
                                message="unclosed",
                                category=ResourceWarning)
        warnings.filterwarnings(action='ignore', message='Unverified HTTPS request')
        func(*args)
    return inner_func

from contextlib import contextmanager

@contextmanager
def created_user(password, _teardown=None):
    from library.user import User

    api = User()
    is_teardown = TEARDOWN
    if _teardown in (True, False):
        is_teardown = _teardown

    user_id, user_name = api.create_user(user_password=password, **ADMIN_CLIENT)
    try:
        yield (user_id, user_name)
    finally:
        if is_teardown:
            api.delete_user(user_id, **ADMIN_CLIENT)

@contextmanager
def created_project(name=None, metadata=None, user_id=None, member_role_id=None, _teardown=None):
    from library.project import Project

    api = Project()

    is_teardown = TEARDOWN
    if _teardown in (True, False):
        is_teardown = _teardown

    project_id, project_name = api.create_project(name=name, metadata=metadata, **ADMIN_CLIENT)
    if user_id:
        api.add_project_members(project_id, user_id=user_id, member_role_id=member_role_id, **ADMIN_CLIENT)

    try:
        yield (project_id, project_name)
    finally:
        if is_teardown:
            api.delete_project(project_id, **ADMIN_CLIENT)



import copy
import os
import subprocess
from g import config_dir, templates_dir, DEFAULT_GID, DEFAULT_UID, data_dir
from urllib.parse import urlsplit, unquote
from utils.jinja import render_jinja
from utils.misc import prepare_dir

registry_config_dir = os.path.join(config_dir, "registry")
registry_config_template_path = os.path.join(templates_dir, "registry", "config.yml.jinja")
registry_conf = os.path.join(config_dir, "registry", "config.yml")
registry_passwd_path = os.path.join(config_dir, "registry", "passwd")
registry_data_dir = os.path.join(data_dir, 'registry')

levels_map = {
    'debug': 'debug',
    'info': 'info',
    'warning': 'warn',
    'error': 'error',
    'fatal': 'fatal'
}

def prepare_registry(config_dict):
    prepare_dir(registry_data_dir, uid=DEFAULT_UID, gid=DEFAULT_GID)
    prepare_dir(registry_config_dir)

    gen_passwd_file(config_dict)
    storage_provider_info = get_storage_provider_info(
        config_dict['storage_provider_name'],
        config_dict['storage_provider_config'])

    # process redis info
    redis_ops = parse_redis(config_dict['redis_url_reg'])

    render_jinja(
        registry_config_template_path,
        registry_conf,
        uid=DEFAULT_UID,
        gid=DEFAULT_GID,
        level=levels_map[config_dict['log_level']],
        storage_provider_info=storage_provider_info,
        **config_dict, **redis_ops)

def parse_redis(redis_url):
    u = urlsplit(redis_url)
    if not u.scheme or u.scheme == 'redis':
        return {
            'redis_host': u.netloc.split('@')[-1],
            'redis_password': '' if u.password is None else unquote(u.password),
            'redis_db_index_reg': u.path and int(u.path[1:]) or 0,
            'redis_enableTLS': 'false',
        }
    elif u.scheme == 'rediss':
        return {
            'redis_host': u.netloc.split('@')[-1],
            'redis_password': '' if u.password is None else unquote(u.password),
            'redis_db_index_reg': u.path and int(u.path[1:]) or 0,
            'redis_enableTLS': 'true',
        }
    elif u.scheme == 'redis+sentinel':
        return {
            'sentinel_master_set': u.path.split('/')[1],
            'redis_host': u.netloc.split('@')[-1],
            'redis_password': '' if u.password is None else unquote(u.password),
            'redis_db_index_reg': len(u.path.split('/')) == 3 and int(u.path.split('/')[2]) or 0,
            'redis_enableTLS': 'false',
        }
    elif u.scheme == 'rediss+sentinel':
        return {
            'sentinel_master_set': u.path.split('/')[1],
            'redis_host': u.netloc.split('@')[-1],
            'redis_password': '' if u.password is None else unquote(u.password),
            'redis_db_index_reg': len(u.path.split('/')) == 3 and int(u.path.split('/')[2]) or 0,
            'redis_enableTLS': 'true',
        }
    else:
        raise Exception('bad redis url for registry:' + redis_url)

def get_storage_provider_info(provider_name, provider_config):
    provider_config_copy = copy.deepcopy(provider_config)
    if provider_name == "filesystem":
        if not (provider_config_copy and ('rootdirectory' in provider_config_copy)):
            provider_config_copy['rootdirectory'] = '/storage'
    if provider_name == 'gcs' and provider_config_copy.get('keyfile'):
        provider_config_copy['keyfile'] = '/etc/registry/gcs.key'
    # generate storage configuration section in yaml format
    storage_provider_conf_list = [provider_name + ':']
    for config in provider_config_copy.items():
        if config[1] is None:
            value = ''
        elif config[1] == True:
            value = 'true'
        else:
            value = config[1]
        storage_provider_conf_list.append('{}: {}'.format(config[0], value))
    storage_provider_info = ('\n' + ' ' * 4).join(storage_provider_conf_list)
    return storage_provider_info

def gen_passwd_file(config_dict):
    return subprocess.call(["/usr/bin/htpasswd", "-bcB", registry_passwd_path, config_dict['registry_username'],
                            config_dict['registry_password']], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)



import base
from v2_swagger_client.rest import ApiException

class QuotaSorting(base.Base):
    def __init__(self):
        super(QuotaSorting,self).__init__(api_type="quota")

    def list_quotas_with_sorting(self, expect_status_code=200, **kwargs):
        params = {}
        if "sort" in kwargs:
            params["sort"] = kwargs["sort"]
        if "reference" in kwargs:
            params["reference"] = kwargs["reference"]

        try:
            resp_data, status_code, _ = self._get_client(**kwargs).list_quotas_with_http_info(**params)
        except ApiException as e:
            raise Exception(r"Error out with exception. Exception status: {}; exception reason: {}; exception body: {}", e.status, e.reason, e.body)
        base._assert_status_code(expect_status_code, status_code)

        return resp_data



# -*- coding: utf-8 -*-

import time
import base
import v2_swagger_client
from v2_swagger_client.rest import ApiException

class StopScan(base.Base, object):
    def __init__(self):
        super(StopScan,self).__init__(api_type = "scan")

    def stop_scan_artifact(self, project_name, repo_name, reference, expect_status_code = 202, expect_response_body = None, **kwargs):
        try:
            scanType = v2_swagger_client.ScanType()
            scanType.scan_type = "vulnerability"
            data, status_code, _ = self._get_client(**kwargs).stop_scan_artifact_with_http_info(project_name, repo_name, reference, scanType)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return

        base._assert_status_code(expect_status_code, status_code)

        return data

    def stop_sbom_generation_of_artifact(self, project_name, repo_name, reference, expect_status_code = 202, expect_response_body = None, **kwargs):
        try:
            scanType = v2_swagger_client.ScanType()
            scanType.scan_type = "sbom"
            data, status_code, _ = self._get_client(**kwargs).stop_scan_artifact_with_http_info(project_name, repo_name, reference, scanType)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return

        base._assert_status_code(expect_status_code, status_code)

        return data


from __future__ import absolute_import

import unittest
import json

from testutils import ADMIN_CLIENT, suppress_urllib3_warning, files_directory
from testutils import TEARDOWN
from testutils import harbor_server
from library.user import User
from library.project import Project
from library.repository import Repository
from library.base import _assert_status_code
from library.repository import push_special_image_to_project, push_self_build_image_to_project
from library.artifact import Artifact
from library.gc import GC
from library import docker_api, cosign


class TestProjects(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.gc = GC()
        self.project = Project()
        self.user = User()
        self.repo = Repository()
        self.artifact = Artifact()
        self.repo_name = "test_repo"
        self.repo_name_untag = "test_untag"
        self.tag = "v1.0"
        self.gc_success_status = "Success"


    def testGarbageCollection(self):
        """
        Test case:
            Garbage Collection
        Test step and expected result:
            1. Create a new user(UA);
            2. Create project(PA) and project(PB) by user(UA);
            3. Push a image in project(PA) and then delete repository by admin;
            4. Get repository by user(UA), it should get nothing;
            5. Push a image in project(PB) by admin and delete the only tag;
            6. Tigger garbage collection operation;
            7. Check garbage collection job was finished;
            8. Get garbage collection log, check there is a number of files was deleted;
            9. Check garbage collection details;
            10. Check the log for garbage collection workers;
            11. Tigger garbage collection operation;
            12. Check garbage collection job was finished;
            13. Get garbage collection log, check there is a number of files was deleted;
            14. Repository with untag image should be still there;
            15. But no any artifact in repository anymore;
            16. Check garbage collection details;
            17. Check the log for garbage collection workers;
        """
        url = ADMIN_CLIENT["endpoint"]
        admin_name = ADMIN_CLIENT["username"]
        admin_password = ADMIN_CLIENT["password"]
        user_gc_password = "Aa123456"

        #1. Create a new user(UA);
        TestProjects.user_gc_id, user_gc_name = self.user.create_user(user_password = user_gc_password, **ADMIN_CLIENT)

        TestProjects.USER_GC_CLIENT=dict(endpoint = url, username = user_gc_name, password = user_gc_password)

        #2. Create project(PA) and project(PB) by user(UA);
        TestProjects.project_gc_id, TestProjects.project_gc_name = self.project.create_project(metadata = {"public": "false"}, **TestProjects.USER_GC_CLIENT)
        TestProjects.project_gc_untag_id, TestProjects.project_gc_untag_name = self.project.create_project(metadata = {"public": "false"}, **TestProjects.USER_GC_CLIENT)

        #3. Push a image in project(PA) and then delete repository by admin;
        push_special_image_to_project(TestProjects.project_gc_name, harbor_server, admin_name, admin_password, self.repo_name, ["latest", "v1.2.3"])
        self.repo.delete_repository(TestProjects.project_gc_name, self.repo_name, **TestProjects.USER_GC_CLIENT)

        #4. Get repository by user(UA), it should get nothing;
        repo_data = self.repo.list_repositories(TestProjects.project_gc_name, **TestProjects.USER_GC_CLIENT)
        _assert_status_code(len(repo_data), 0)

        #5. Push a image in project(PB) by admin and delete the only tag;
        push_special_image_to_project(TestProjects.project_gc_untag_name, harbor_server, admin_name, admin_password, self.repo_name_untag, [self.tag])
        self.artifact.delete_tag(TestProjects.project_gc_untag_name, self.repo_name_untag, self.tag, self.tag, **ADMIN_CLIENT)

        #6. Tigger garbage collection operation;
        gc_id = self.gc.gc_now(**ADMIN_CLIENT)

        #7. Check garbage collection job was finished;
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        #8. Get garbage collection log, check there is a number of files was deleted;
        self.gc.validate_deletion_success(gc_id, **ADMIN_CLIENT)
        artifacts = self.artifact.list_artifacts(TestProjects.project_gc_untag_name, self.repo_name_untag, **TestProjects.USER_GC_CLIENT)
        _assert_status_code(len(artifacts), 1)

        #9. Check garbage collection details;
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], False, False, 1000000, 1100000, 2, 1, 1)

        #10. Check the log for garbage collection workers;
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: 1", gc_log)

        #11. Tigger garbage collection operation;
        workers = 2
        gc_id = self.gc.gc_now(is_delete_untagged=True, workers=2, **ADMIN_CLIENT)

        #12. Check garbage collection job was finished;
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        #13. Get garbage collection log, check there is a number of files was deleted;
        self.gc.validate_deletion_success(gc_id, **ADMIN_CLIENT)

        #14. Repository with untag image should be still there;
        repo_data_untag = self.repo.list_repositories(TestProjects.project_gc_untag_name, **TestProjects.USER_GC_CLIENT)
        _assert_status_code(len(repo_data_untag), 1)
        self.assertEqual(TestProjects.project_gc_untag_name + "/" + self.repo_name_untag , repo_data_untag[0].name)

        #15. But no any artifact in repository anymore.
        artifacts = self.artifact.list_artifacts(TestProjects.project_gc_untag_name, self.repo_name_untag, **TestProjects.USER_GC_CLIENT)
        self.assertEqual(artifacts,[])

        #16. Check garbage collection details;
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], True, False, 1500000, 2000000, 3, 1, workers)

        #17. Check the log for garbage collection workers;
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)


    def testGarbageCollectionAccessory(self):
        """
        Test case:
            Garbage Collection Accessory
        Test step and expected result:
            1. Create a new user(UA);
            2. Create a new project(PA) by user(UA);
            3. Push a new image(IA) in project(PA) by user(UA);
            4. Push image(IA) SBOM to project(PA) by user(UA);
            5. Sign image(IA) with cosign;
            6. Sign image(IA) SBOM with cosign;
            7. Sign image(IA) Signature with cosign;
            8. Delete image(IA) Signature of signature by user(UA);
            9. Trigger GC and wait for GC to succeed;
            10. Check garbage collection details;
            11. Check the log for garbage collection workers;
            12. Get the GC log and check that the image(IA) Signature of signature is deleted;
            13. Delete image(IA) Signature by user(UA);
            14. Trigger GC and wait for GC to succeed;
            15. Check garbage collection details;
            16. Check the log for garbage collection workers;
            17. Get the GC log and check that the image(IA) Signature is deleted;
            18. Delete image(IA) SBOM by user(UA);
            19. Trigger GC and wait for GC to succeed;
            20. Check garbage collection details;
            21. Check the log for garbage collection workers;
            22. Get the GC log and check that the image(IA) SBOM and Signature of SBOM is deleted;
            23. Push image(IA) SBOM to project(PA) by user(UA);
            24. Sign image(IA) with cosign;
            25. Sign image(IA) SBOM with cosign;
            26. Sign image(IA) Signature with cosign;
            27. Trigger GC and wait for GC to succeed;
            28. Check garbage collection details;
            29. Check the log for garbage collection workers;
            30. Get the GC log and check that it is not deleted;
            31. Delete tag of image(IA) by user(UA);
            32. Trigger GC and wait for GC to succeed;
            33. Check garbage collection details;
            34. Check the log for garbage collection workers;
            35. Get the GC log and check that the image(IA) and all aeecssory is deleted;
        """
        url = ADMIN_CLIENT["endpoint"]
        user_password = "Aa123456"
        deleted_prefix =  "delete blob from storage: "
        # 1. Create user(UA)
        _, user_name = self.user.create_user(user_password = user_password, **ADMIN_CLIENT)
        user_client = dict(endpoint = url, username = user_name, password = user_password, with_accessory = True)

        # 2. Create private project(PA) by user(UA)
        self.image = "test_image"
        _, project_name = self.project.create_project(metadata = {"public": "false"}, **user_client)

        # 3. Push a new image(IA) in project(PA) by user(UA)
        push_self_build_image_to_project(project_name, harbor_server, user_name, user_password, self.image, self.tag)

        # 4. Push image(IA) SBOM to project(PA) by user(UA)
        self.sbom_path = files_directory + "sbom_test.json"
        docker_api.docker_login_cmd(harbor_server, user_name, user_password, enable_manifest = False)
        cosign.push_artifact_sbom("{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag), self.sbom_path)
        artifact_info = self.artifact.get_reference_info(project_name, self.image, self.tag, **user_client)
        sbom_digest = artifact_info.accessories[0].digest

        # 5. Sign image(IA) with cosign
        cosign.generate_key_pair()
        cosign.sign_artifact("{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag))
        artifact_info = self.artifact.get_reference_info(project_name, self.image, self.tag, **user_client)
        self.assertEqual(len(artifact_info.accessories), 2)
        image_signature_digest = None
        for accessory in artifact_info.accessories:
            if accessory.digest != sbom_digest:
                image_signature_digest = accessory.digest
                break

        # 6. Sign image(IA) SBOM cosign
        cosign.sign_artifact("{}/{}/{}@{}".format(harbor_server, project_name, self.image, sbom_digest))
        sbom_info = self.artifact.get_reference_info(project_name, self.image, sbom_digest, **user_client)
        sbom_signature_digest = sbom_info.accessories[0].digest

        # 7. Sign image(IA) Signature with cosign
        cosign.sign_artifact("{}/{}/{}@{}".format(harbor_server, project_name, self.image, image_signature_digest))
        signature_info = self.artifact.get_reference_info(project_name, self.image, image_signature_digest, **user_client)
        signature_signature_digest = signature_info.accessories[0].digest

        # 8. Delete image(IA) Signature of signature by user(UA)
        self.artifact.delete_artifact(project_name, self.image, signature_signature_digest, **user_client)

        # 9. Trigger GC and wait for GC to succeed
        workers = 3
        gc_id = self.gc.gc_now(workers=3,**ADMIN_CLIENT)
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        # 10. Check garbage collection details
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], False, False, 1500, 3000, 2, 1, workers)

        # 11. Check the log for garbage collection workers
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)

        # 12. Get the GC log and check that the image(IA) Signature of signature is deleted
        self.assertIn(deleted_prefix + signature_signature_digest, gc_log)

        # 13. Delete image(IA) Signature by user(UA)
        self.artifact.delete_artifact(project_name, self.image, image_signature_digest, **user_client)

        # 14. Trigger GC and wait for GC to succeed
        workers = 4
        gc_id = self.gc.gc_now(workers=workers, **ADMIN_CLIENT)
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        # 15. Check garbage collection details
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], False, False, 1500, 3000, 2, 1, workers)

        # 16. Check the log for garbage collection workers
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)

        # 17. Get the GC log and check that the image(IA) Signature is deleted
        self.assertIn(deleted_prefix + image_signature_digest, gc_log)

        # 18. Delete image(IA) SBOM by user(UA)
        self.artifact.delete_artifact(project_name, self.image, sbom_digest, **user_client)

        # 19. Trigger GC and wait for GC to succeed
        workers = 5
        gc_id = self.gc.gc_now(workers=workers, **ADMIN_CLIENT)
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        # 20. Check garbage collection details
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], False, False, 4000, 5000, 4, 2, workers)

        # 21. Check the log for garbage collection workers
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)

        # 22. Get the GC log and check that the image(IA) SBOM and Signature of SBOM is deleted
        self.assertIn(deleted_prefix + sbom_digest, gc_log)
        self.assertIn(deleted_prefix + sbom_signature_digest, gc_log)

        # 23. Push image(IA) SBOM to project(PA) by user(UA)
        self.sbom_path = files_directory + "sbom_test.json"
        docker_api.docker_login_cmd(harbor_server, user_name, user_password, enable_manifest = False)
        cosign.push_artifact_sbom("{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag), self.sbom_path)
        artifact_info = self.artifact.get_reference_info(project_name, self.image, self.tag, **user_client)
        sbom_digest = artifact_info.accessories[0].digest

        # 24. Sign image(IA) with cosign
        cosign.sign_artifact("{}/{}/{}:{}".format(harbor_server, project_name, self.image, self.tag))
        artifact_info = self.artifact.get_reference_info(project_name, self.image, self.tag, **user_client)
        self.assertEqual(len(artifact_info.accessories), 2)
        image_signature_digest = None
        for accessory in artifact_info.accessories:
            if accessory.digest != sbom_digest:
                image_signature_digest = accessory.digest
                break

        # 25. Sign image(IA) SBOM cosign
        cosign.sign_artifact("{}/{}/{}@{}".format(harbor_server, project_name, self.image, sbom_digest))
        sbom_info = self.artifact.get_reference_info(project_name, self.image, sbom_digest, **user_client)
        sbom_signature_digest = sbom_info.accessories[0].digest

        # 26. Sign image(IA) Signature with cosign
        cosign.sign_artifact("{}/{}/{}@{}".format(harbor_server, project_name, self.image, image_signature_digest))
        signature_info = self.artifact.get_reference_info(project_name, self.image, image_signature_digest, **user_client)
        signature_signature_digest = signature_info.accessories[0].digest

        # 27. Trigger GC and wait for GC to succeed
        workers = 1
        gc_id = self.gc.gc_now(**ADMIN_CLIENT)
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        # 28. Check garbage collection details
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], False, False, 0, 0, 0, 0, workers)

        # 29. Check the log for garbage collection workers
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)

        # 30. Get the GC log and check that it is not deleted
        self.assertNotIn(deleted_prefix + sbom_digest, gc_log)
        self.assertNotIn(deleted_prefix + sbom_signature_digest, gc_log)
        self.assertNotIn(deleted_prefix + image_signature_digest, gc_log)
        self.assertNotIn(deleted_prefix + signature_signature_digest, gc_log)

        # 31. Delete tag of image(IA) by user(UA)
        self.artifact.delete_tag(project_name, self.image, self.tag, self.tag, **user_client)

        # 32. Trigger GC and wait for GC to succeed
        workers = 2
        gc_id = self.gc.gc_now(workers=workers, is_delete_untagged=True, **ADMIN_CLIENT)
        self.gc.validate_gc_job_status(gc_id, self.gc_success_status, **ADMIN_CLIENT)

        # 33. Check garbage collection details
        gc_history = self.gc.get_gc_history(**ADMIN_CLIENT)
        self.checkGarbageCollectionDetails(gc_history[0], True, False, 2500000, 3000000, 11, 5, workers)

        # 34. Check the log for garbage collection workers
        gc_log = self.gc.get_gc_log_by_id(gc_id, **ADMIN_CLIENT)
        self.assertIn("workers: {}".format(workers), gc_log)

        # 35. Get the GC log and check that the image(IA) and all aeecssory is deleted
        self.assertIn(self.image, gc_log)
        self.assertIn(deleted_prefix + sbom_digest, gc_log)
        self.assertIn(deleted_prefix + sbom_signature_digest, gc_log)
        self.assertIn(deleted_prefix + image_signature_digest, gc_log)
        self.assertIn(deleted_prefix + signature_signature_digest, gc_log)


    def checkGarbageCollectionDetails(self, gc, delete_untagged, dry_run,
                                      freed_space_range_start, freed_space_range_end,
                                      purged_blobs, purged_manifests, workers):
        self.assertEqual(gc.job_kind, "MANUAL")
        self.assertEqual(gc.job_name, "GARBAGE_COLLECTION")
        parameters = json.loads(gc.job_parameters)
        self.assertEqual(parameters["delete_untagged"], delete_untagged)
        self.assertEqual(parameters["dry_run"], dry_run)
        self.assertTrue(parameters["freed_space"] >= freed_space_range_start)
        self.assertTrue(parameters["freed_space"] <= freed_space_range_end)
        self.assertEqual(parameters["purged_blobs"], purged_blobs)
        self.assertEqual(parameters["purged_manifests"], purged_manifests)
        self.assertEqual(parameters["workers"], workers)


if __name__ == '__main__':
    unittest.main()



import os

from g import templates_dir
from .configs import parse_versions
from .jinja import render_jinja

docker_compose_template_path = os.path.join(templates_dir, 'docker_compose', 'docker-compose.yml.jinja')
docker_compose_yml_path = '/var/lib/harbor/docker-compose.yml'

# render docker-compose
def prepare_docker_compose(configs, with_trivy):
    versions = parse_versions()
    VERSION_TAG = versions.get('VERSION_TAG') or 'latest'
    REDIS_VERSION = versions.get('REDIS_VERSION') or 'latest'
    NGINX_VERSION = versions.get('NGINX_VERSION') or 'latest'
    TRIVY_ADAPTER_VERSION = versions.get('TRIVY_ADAPTER_VERSION') or 'latest'
    REGISTRY_VERSION = versions.get('REGISTRY_VERSION') or 'latest'

    rendering_variables = {
        'image_src': configs['image_src'],
        'version': VERSION_TAG,
        'reg_version': REGISTRY_VERSION,
        'redis_version': REDIS_VERSION,
        'nginx_version': NGINX_VERSION,
        'trivy_adapter_version': TRIVY_ADAPTER_VERSION,
        'data_volume': configs['data_volume'],
        'log_location': configs['log_location'],
        'protocol': configs['protocol'],
        'http_port': configs['http_port'],
        'external_redis': configs['external_redis'],
        'external_database': configs['external_database'],
        'with_trivy': with_trivy,
    }

    # if configs.get('registry_custom_ca_bundle_path'):
    #     rendering_variables['registry_custom_ca_bundle_path'] = configs.get('registry_custom_ca_bundle_path')
    #     rendering_variables['custom_ca_required'] = True

    # for gcs
    storage_config = configs.get('storage_provider_config') or {}
    if storage_config.get('keyfile') and configs['storage_provider_name'] == 'gcs':
        rendering_variables['gcs_keyfile'] = storage_config['keyfile']

    # for http
    if configs['protocol'] == 'https':
        rendering_variables['cert_key_path'] = configs['cert_key_path']
        rendering_variables['cert_path'] = configs['cert_path']
        rendering_variables['https_port'] = configs['https_port']

    # internal cert pairs
    rendering_variables['internal_tls'] = configs['internal_tls']

    # for uaa
    uaa_config = configs.get('uaa') or {}
    if uaa_config.get('ca_file'):
        rendering_variables['uaa_ca_file'] = uaa_config['ca_file']

    # for log
    log_ep_host = configs.get('log_ep_host')
    if log_ep_host:
        rendering_variables['external_log_endpoint'] = True

    # for metrics
    metric = configs.get('metric')
    if metric:
        rendering_variables['metric'] = metric

    render_jinja(docker_compose_template_path, docker_compose_yml_path,  mode=0o644, **rendering_variables)



from __future__ import absolute_import


import unittest

from testutils import ADMIN_CLIENT, suppress_urllib3_warning
from testutils import harbor_server
from testutils import TEARDOWN
from library.base import _assert_status_code
from library.project import Project
from library.user import User
from library.repository import Repository
from library.repository import push_self_build_image_to_project

class TestProjects(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.project= Project()
        self.user= User()
        self.repo= Repository()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        #1. Delete project(PA);
        self.project.delete_project(TestProjects.project_del_repo_id, **TestProjects.USER_del_repo_CLIENT)

        #2. Delete user(UA).
        self.user.delete_user(TestProjects.user_del_repo_id, **ADMIN_CLIENT)

    def testDelRepo(self):
        """
        Test case:
            Delete a repository
        Test step and expected result:
            1. Create a new user(UA);
            2. Create a new project(PA) by user(UA);
            3. Create a new repository(RA) in project(PA) by user(UA);
            4. Get repository in project(PA), there should be one repository which was created by user(UA);
            5. Delete repository(RA) by user(UA);
            6. Get repository by user(UA), it should get nothing;
        Tear down:
            1. Delete project(PA);
            2. Delete user(UA).
        """
        url = ADMIN_CLIENT["endpoint"]
        user_del_repo_password = "Aa123456"

        #1. Create a new user(UA);
        TestProjects.user_del_repo_id, user_del_repo_name = self.user.create_user(user_password = user_del_repo_password, **ADMIN_CLIENT)

        TestProjects.USER_del_repo_CLIENT=dict(endpoint = url, username = user_del_repo_name, password = user_del_repo_password)

        #2. Create a new project(PA) by user(UA);
        TestProjects.project_del_repo_id, TestProjects.project_del_repo_name = self.project.create_project(metadata = {"public": "false"}, **TestProjects.USER_del_repo_CLIENT)

        #3. Create a new repository(RA) in project(PA) by user(UA);
        repo_name, _ = push_self_build_image_to_project(TestProjects.project_del_repo_name, harbor_server, 'admin', 'Harbor12345', "test_del_repo", "latest", size=512)

        #4. Get repository in project(PA), there should be one repository which was created by user(UA);
        repo_data = self.repo.list_repositories(TestProjects.project_del_repo_name, **TestProjects.USER_del_repo_CLIENT)
        _assert_status_code(repo_name, repo_data[0].name)

        #5. Delete repository(RA) by user(UA);
        self.repo.delete_repository(TestProjects.project_del_repo_name, repo_name.split('/')[1], **TestProjects.USER_del_repo_CLIENT)

        #6. Get repository by user(UA), it should get nothing;
        repo_data = self.repo.list_repositories(TestProjects.project_del_repo_name, **TestProjects.USER_del_repo_CLIENT)
        _assert_status_code(len(repo_data), 0)

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

import time
import base
import importlib
import docker_api
from docker_api import DockerAPI
# from v2_swagger_client.rest import ApiException
from testutils import DOCKER_USER, DOCKER_PWD

def swagger_module():
        module = importlib.import_module("v2_swagger_client")
        return module

def pull_harbor_image(registry, username, password, image, tag, expected_login_error_message = None, expected_error_message = None):
    _docker_api = DockerAPI()
    _docker_api.docker_login(registry, username, password, expected_error_message = expected_login_error_message)
    if expected_login_error_message != None:
        return
    time.sleep(2)
    _docker_api.docker_image_pull(r'{}/{}'.format(registry, image), tag = tag, expected_error_message = expected_error_message)

def push_image_to_project(project_name, registry, username, password, image, tag, expected_login_error_message = None, expected_error_message = None, profix_for_image = None, new_image=None):
    _docker_api = DockerAPI()
    _docker_api.docker_image_pull(image, tag, is_clean_all_img = False)
    _docker_api.docker_login(registry, username, password, expected_error_message = expected_login_error_message)
    time.sleep(2)
    if expected_login_error_message != None:
        return
    time.sleep(2)
    original_name = image
    image = new_image or image

    if profix_for_image == None:
        target_image = r'{}/{}/{}'.format(registry, project_name, image)
    else:
        target_image = r'{}/{}/{}/{}'.format(registry, project_name, profix_for_image, image)
    new_harbor_registry, new_tag = _docker_api.docker_image_tag(r'{}:{}'.format(original_name, tag), target_image, tag = tag)
    time.sleep(2)
    _docker_api.docker_image_push(new_harbor_registry, new_tag, expected_error_message = expected_error_message)
    docker_api.docker_image_clean_all()
    return r'{}/{}'.format(project_name, image), new_tag

def push_self_build_image_to_project(project_name, registry, username, password, image, tag, size=2, expected_login_error_message = None, expected_error_message = None):
    _docker_api = DockerAPI()
    _docker_api.docker_login(registry, username, password, expected_error_message = expected_login_error_message)

    time.sleep(2)
    if expected_login_error_message in [None, ""]:
        push_special_image_to_project(project_name, registry, username, password, image, tags=[tag], size=size, expected_login_error_message = expected_login_error_message, expected_error_message = expected_error_message)
        return r'{}/{}'.format(project_name, image), tag

def push_special_image_to_project(project_name, registry, username, password, image, tags=None, size=1, expected_login_error_message=None, expected_error_message = None):
    _docker_api = DockerAPI()
    _docker_api.docker_login(registry, username, password, expected_error_message = expected_login_error_message)
    time.sleep(2)
    if expected_login_error_message in [None, ""]:
        return _docker_api.docker_image_build(r'{}/{}/{}'.format(registry, project_name, image), tags = tags, size=int(size), expected_error_message=expected_error_message, clean_images=False)

def push_local_image_to_project(registry, username, password, original_image, original_tag, target_image, target_tag):
    _docker_api = DockerAPI()
    _docker_api.docker_login(registry, username, password)
    new_harbor_registry, new_tag = _docker_api.docker_image_tag(r'{}:{}'.format(original_image, original_tag), target_image, tag = target_tag)
    _docker_api.docker_image_push(new_harbor_registry, new_tag)

class Repository(base.Base, object):
    def __init__(self):
        super(Repository,self).__init__(api_type = "repository")

    def list_tags(self, repository, **kwargs):
        client = self._get_client(**kwargs)
        return client.repositories_repo_name_tags_get(repository)

    def get_tag(self, repo_name, tag, **kwargs):
        client = self._get_client(**kwargs)
        return client.repositories_repo_name_tags_tag_get(repo_name, tag)

    def image_exists(self, repository, tag, **kwargs):
        tags = self.list_tags(repository, **kwargs)
        exist = False
        for t in tags:
            if t.name == tag:
                exist = True
                break
        return exist

    def image_should_exist(self, repository, tag, **kwargs):
        if not self.image_exists(repository, tag, **kwargs):
            raise Exception("image %s:%s not exist" % (repository, tag))

    def image_should_not_exist(self, repository, tag, **kwargs):
        if self.image_exists(repository, tag, **kwargs):
            raise Exception("image %s:%s exists" % (repository, tag))

    def delete_repository(self, project_name, repo_name, expect_status_code = 200, expect_response_body = None, **kwargs):
        client = self._get_client(**kwargs)
        try:
            _, status_code, _ = client.delete_repository_with_http_info(project_name, repo_name)
        except Exception as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return
        else:
            base._assert_status_code(expect_status_code, status_code)
            base._assert_status_code(200, status_code)


    def list_repositories(self, project_name, **kwargs):
        client = self._get_client(**kwargs)
        data, status_code, _ = client.list_repositories_with_http_info(project_name)
        base._assert_status_code(200, status_code)
        return data

    def clear_repositories(self, project_name, **kwargs):
        repos = self.list_repositories(project_name, **kwargs)
        print("Repos to be cleared:", repos)
        for repo in repos:
            self.delete_repository(project_name, repo.name.split('/')[1])

    def get_repository(self, project_name, repo_name, **kwargs):
        client = self._get_client(**kwargs)
        data, status_code, _ = client.get_repository_with_http_info(project_name, repo_name)
        base._assert_status_code(200, status_code)
        return data

    def add_label_to_tag(self, repo_name, tag, label_id, expect_status_code = 200, **kwargs):
        client = self._get_client(**kwargs)
        v2_swagger_client = swagger_module()
        label = v2_swagger_client.Label(id=label_id)
        _, status_code, _ = client.repositories_repo_name_tags_tag_labels_post_with_http_info(repo_name, tag, label)
        base._assert_status_code(expect_status_code, status_code)

    def get_repo_signatures(self, repo_name, expect_status_code = 200, **kwargs):
        client = self._get_client(**kwargs)
        data, status_code, _ = client.repositories_repo_name_signatures_get_with_http_info(repo_name)
        base._assert_status_code(expect_status_code, status_code)
        return data

    def check_image_not_scanned(self, repo_name, tag, **kwargs):
        tag = self.get_tag(repo_name, tag, **kwargs)
        if tag.scan_overview != None:
            raise Exception("Image should be <Not Scanned> state!")

    def repository_should_exist(self, project_Name, repo_name, **kwargs):
        repositories = self.list_repositories(project_Name, **kwargs)
        if repo_name not in repositories:
            raise Exception("Repository {} is not exist.".format(repo_name))

    def check_repository_exist(self, project_Name, repo_name, **kwargs):
        repositories = self.list_repositories(project_Name, **kwargs)
        for  repo in repositories:
            if repo.name == project_Name+"/"+repo_name:
                return True
        return False

    def signature_should_exist(self, repo_name, tag, **kwargs):
        signatures = self.get_repo_signatures(repo_name, **kwargs)
        for each_sign in signatures:
            if each_sign.tag == tag and len(each_sign.hashes["sha256"]) == 44:
                return
        raise Exception(r"Signature of {}:{} is not exist!".format(repo_name, tag))

    def retag_image(self, repo_name, tag, src_image, override=True, expect_status_code = 200, expect_response_body = None, **kwargs):
        client = self._get_client(**kwargs)
        v2_swagger_client = swagger_module()
        request = v2_swagger_client.RetagReq(tag=tag, src_image=src_image, override=override)

        try:
            data, status_code, _ = client.repositories_repo_name_tags_post_with_http_info(repo_name, request)
        except v2_swagger_client.rest.ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return

        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(200, status_code)
        return data



# -*- coding: utf-8 -*-
import base
import os

def generate_key_pair():
    config_key_file = "cosign.key"
    config_pub_file = "cosign.pub"
    if os.path.exists(config_key_file) and os.path.exists(config_pub_file):
        os.remove(config_key_file)
        os.remove(config_pub_file)
    command = ["cosign", "generate-key-pair"]
    base.run_command(command)

def sign_artifact(artifact):
    command = ["cosign", "sign", "-y", "--allow-insecure-registry", "--key", "cosign.key", artifact]
    base.run_command(command)

def push_artifact_sbom(artifact, sbom_path, type="spdx"):
    command = ["cosign", "attach", "sbom", "--allow-insecure-registry", "--registry-referrers-mode", "oci-1-1",
               "--type", type, "--sbom", sbom_path, artifact]
    base.run_command(command)



# -*- coding: utf-8 -*-

import time
import base
import v2_swagger_client

class Replication(base.Base, object):
    def __init__(self):
        super(Replication,self).__init__(api_type = "replication")

    def wait_until_jobs_finish(self, rule_id, retry=10, interval=5, **kwargs):
        Succeed = False
        for i in range(retry):
            Succeed = False
            jobs = self.get_replication_executions(rule_id, **kwargs)
            for job in jobs:
                if job.status == "Succeed":
                    return
            if not Succeed:
                time.sleep(interval)
        if not Succeed:
            raise Exception("The jobs not Succeed")

    def trigger_replication_executions(self, rule_id, expect_status_code = 201, **kwargs):
        _, status_code, _ = self._get_client(**kwargs).start_replication_with_http_info({"policy_id":rule_id})
        base._assert_status_code(expect_status_code, status_code)

    def get_replication_executions(self, rule_id, expect_status_code = 200, **kwargs):
        data, status_code, _ = self._get_client(**kwargs).list_replication_executions_with_http_info(policy_id=rule_id)
        base._assert_status_code(expect_status_code, status_code)
        return data

    def create_replication_policy(self, dest_registry=None, src_registry=None, name=None, description="",
                                  dest_namespace = "", filters=None, trigger=v2_swagger_client.ReplicationTrigger(type="manual",trigger_settings=v2_swagger_client.ReplicationTriggerSettings(cron="")),
                                  deletion=False, override=True, enabled=True, expect_status_code = 201, **kwargs):
        if name is None:
            name = base._random_name("rule")
        if filters is None:
            filters = []

        policy = v2_swagger_client.ReplicationPolicy(name=name, description=description,dest_namespace=dest_namespace,
                                                  dest_registry=dest_registry, src_registry=src_registry,filters=filters,
                                                  trigger=trigger, deletion=deletion, override=override, enabled=enabled)
        _, status_code, header = self._get_client(**kwargs).create_replication_policy_with_http_info(policy)
        base._assert_status_code(expect_status_code, status_code)
        return base._get_id_from_header(header), name

    def get_replication_rule(self, param = None, rule_id = None, expect_status_code = 200, **kwargs):
        if rule_id is None:
            if param is None:
                param = dict()
            data, status_code, _ = self._get_client(**kwargs).get_replication_policy_with_http_info(param)
        else:
            data, status_code, _ = self._get_client(**kwargs).get_replication_policy_with_http_info(rule_id)
        base._assert_status_code(expect_status_code, status_code)
        return data

    def check_replication_rule_should_exist(self, check_rule_id, expect_rule_name, expect_trigger = None, **kwargs):
        rule_data = self.get_replication_rule(rule_id = check_rule_id, **kwargs)
        if str(rule_data.name) != str(expect_rule_name):
            raise Exception(r"Check replication rule failed, expect <{}> actual <{}>.".format(expect_rule_name, str(rule_data.name)))
        else:
            print(r"Check Replication rule passed, rule name <{}>.".format(str(rule_data.name)))
            #get_trigger = str(rule_data.trigger.kind)
            #if expect_trigger is not None and get_trigger == str(expect_trigger):
            #    print r"Check Replication rule trigger passed, trigger name <{}>.".format(get_trigger)
            #else:
            #    raise Exception(r"Check replication rule trigger failed, expect <{}> actual <{}>.".format(expect_trigger, get_trigger))

    def delete_replication_rule(self, rule_id, expect_status_code = 200, **kwargs):
        _, status_code, _ = self._get_client(**kwargs).delete_replication_policy_with_http_info(rule_id)
        base._assert_status_code(expect_status_code, status_code)




from __future__ import absolute_import
import unittest

from testutils import harbor_server, suppress_urllib3_warning
from testutils import TEARDOWN
from testutils import ADMIN_CLIENT
from library.project import Project
from library.user import User
from library.repository import Repository
from library.repository import push_self_build_image_to_project
from library.artifact import Artifact
from library.scan_all import ScanAll

class TestScanAll(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.project= Project()
        self.user= User()
        self.artifact = Artifact()
        self.repo = Repository()
        self.scan_all = ScanAll()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        #1. Delete Alice's repository and Luca's repository;
        self.repo.delete_repository(TestScanAll.project_Alice_name, TestScanAll.repo_Alice_name.split('/')[1], **ADMIN_CLIENT)
        self.repo.delete_repository(TestScanAll.project_Luca_name, TestScanAll.repo_Luca_name.split('/')[1], **ADMIN_CLIENT)

        #2. Delete Alice's project and Luca's project;
        self.project.delete_project(TestScanAll.project_Alice_id, **ADMIN_CLIENT)
        self.project.delete_project(TestScanAll.project_Luca_id, **ADMIN_CLIENT)

        #3. Delete user Alice and Luca.
        self.user.delete_user(TestScanAll.user_Alice_id, **ADMIN_CLIENT)
        self.user.delete_user(TestScanAll.user_Luca_id, **ADMIN_CLIENT)
        print("Case completed")

    def testSystemLevelScanALL(self):
        """
        Test case:
            System level Scan All
        Test step and expected result:
            1. Create user Alice and Luca;
            2. Create 2 new private projects project_Alice and project_Luca;
            3. Push a image to project_Alice and push another image to project_Luca;
            4. Trigger scan all event;
            5. Check if image in project_Alice and another image in project_Luca were both scanned.
        Tear down:
            1. Delete Alice's repository and Luca's repository;
            2. Delete Alice's project and Luca's project;
            3. Delete user Alice and Luca.
        """
        url = ADMIN_CLIENT["endpoint"]
        user_common_password = "Aa123456"

        #1. Create user Alice and Luca;
        TestScanAll.user_Alice_id, user_Alice_name = self.user.create_user(user_password = user_common_password, **ADMIN_CLIENT)
        TestScanAll.user_Luca_id, user_Luca_name = self.user.create_user(user_password = user_common_password, **ADMIN_CLIENT)

        USER_ALICE_CLIENT=dict(endpoint = url, username = user_Alice_name, password = user_common_password, with_scan_overview = True)
        USER_LUCA_CLIENT=dict(endpoint = url, username = user_Luca_name, password = user_common_password, with_scan_overview = True)

        #2. Create 2 new private projects project_Alice and project_Luca;
        TestScanAll.project_Alice_id, TestScanAll.project_Alice_name = self.project.create_project(metadata = {"public": "false"}, **USER_ALICE_CLIENT)
        TestScanAll.project_Luca_id, TestScanAll.project_Luca_name = self.project.create_project(metadata = {"public": "false"}, **USER_LUCA_CLIENT)

        #3. Push a image to project_Alice and push another image to project_Luca;

        #Note: Please make sure that this Image has never been pulled before by any other cases,
        #          so it is a not-scanned image rigth after repository creation.
        #image = "tomcat"
        image_a = "mariadb"
        src_tag = "latest"
        #3.1 Push a image to project_Alice;
        TestScanAll.repo_Alice_name, tag_Alice = push_self_build_image_to_project(TestScanAll.project_Alice_name, harbor_server, user_Alice_name, user_common_password, image_a, src_tag)

        #Note: Please make sure that this Image has never been pulled before by any other cases,
        #          so it is a not-scanned image rigth after repository creation.
        image_b = "httpd"
        src_tag = "latest"
        #3.2 push another image to project_Luca;
        TestScanAll.repo_Luca_name, tag_Luca = push_self_build_image_to_project(TestScanAll.project_Luca_name, harbor_server, user_Luca_name, user_common_password, image_b, src_tag)

        #4. Trigger scan all event;
        self.scan_all.scan_all_now(**ADMIN_CLIENT)
        self.scan_all.wait_until_scans_all_finish(**ADMIN_CLIENT)

        #5. Check if image in project_Alice and another image in project_Luca were both scanned.
        self.artifact.check_image_scan_result(TestScanAll.project_Alice_name, image_a, tag_Alice, **USER_ALICE_CLIENT)
        self.artifact.check_image_scan_result(TestScanAll.project_Luca_name, image_b, tag_Luca, **USER_LUCA_CLIENT)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import base
import v2_swagger_client

class Retention(base.Base):
    def __init__(self):
        super(Retention,self).__init__(api_type="retention")

    def get_metadatas(self, expect_status_code = 200, **kwargs):
        metadatas, status_code, _ = self._get_client(**kwargs).get_rentenition_metadata_with_http_info()
        base._assert_status_code(expect_status_code, status_code)
        return metadatas

    def create_retention_policy(self, project_id, selector_repository="**", selector_tag="**", expect_status_code = 201, **kwargs):
        policy=v2_swagger_client.RetentionPolicy(
            algorithm='or',
            rules=[
                v2_swagger_client.RetentionRule(
                    disabled=False,
                    action="retain",
                    template="always",
                    params= {

                    },
                    scope_selectors={
                        "repository": [
                            {
                                "kind": "doublestar",
                                "decoration": "repoMatches",
                                "pattern": selector_repository
                            }
                        ]
                    },
                    tag_selectors=[
                        {
                            "kind": "doublestar",
                            "decoration": "matches",
                            "pattern": selector_tag
                        }
                    ]
                )
            ],
            trigger= {
                "kind": "Schedule",
                "settings": {
                    "cron": ""
                },
                "references": {
                }
            },
            scope= {
                "level": "project",
                "ref": project_id
            },
        )
        _, status_code, header = self._get_client(**kwargs).create_retention_with_http_info(policy)
        base._assert_status_code(expect_status_code, status_code)
        return base._get_id_from_header(header)

    def get_retention_policy(self, retention_id, expect_status_code = 200, **kwargs):
        policy, status_code, _ = self._get_client(**kwargs).get_retention_with_http_info(retention_id)
        base._assert_status_code(expect_status_code, status_code)
        return policy

    def update_retention_policy(self, retention_id, project_id, selector_repository="**", selector_tag="**", cron="", expect_status_code = 200, **kwargs):
        policy=v2_swagger_client.RetentionPolicy(
            id=retention_id,
            algorithm='or',
            rules=[
                v2_swagger_client.RetentionRule(
                    disabled=False,
                    action="retain",
                    template="always",
                    params= {},
                    scope_selectors={
                        "repository": [
                            {
                                "kind": "doublestar",
                                "decoration": "repoMatches",
                                "pattern": selector_repository
                            }
                        ]
                    },
                    tag_selectors=[
                        {
                            "kind": "doublestar",
                            "decoration": "matches",
                            "pattern": selector_tag
                        }
                    ]
                )
            ],
            trigger= {
                "kind": "Schedule",
                "settings": {
                    "cron": cron
                },
                "references": {
                }
            },
            scope= {
                "level": "project",
                "ref": project_id
            },
        )
        _, status_code, _ = self._get_client(**kwargs).update_retention_with_http_info(retention_id, policy)
        base._assert_status_code(expect_status_code, status_code)

    def update_retention_add_rule(self, retention_id, selector_repository="**", selector_tag="**", with_untag="True", expect_status_code = 200, **kwargs):
        retention_rule = v2_swagger_client.RetentionRule(
                            disabled=False,
                            action="retain",
                            template="always",
                            params= {

                            },
                            scope_selectors={
                                "repository": [
                                    {
                                        "kind": "doublestar",
                                        "decoration": "repoMatches",
                                        "pattern": selector_repository
                                    }
                                ]
                            },
                            tag_selectors=[
                                {
                                    "kind": "doublestar",
                                    "decoration": "matches",
                                    "extras":'["untagged":'+with_untag+']',
                                    "pattern": selector_tag
                                }
                            ]
                        )
        client = self._get_client(**kwargs)
        policy, status_code, _ = client.get_retention_with_http_info(retention_id)
        base._assert_status_code(200, status_code)
        policy.rules.append(retention_rule)
        _, status_code, _ = client.update_retention_with_http_info(retention_id, policy)
        base._assert_status_code(expect_status_code, status_code)

    def trigger_retention_policy(self, retention_id, dry_run=False, expect_status_code = 201, **kwargs):
        _, status_code, _ = self._get_client(**kwargs).trigger_retention_execution_with_http_info(retention_id, {"dry_run":dry_run})
        base._assert_status_code(expect_status_code, status_code)

    def stop_retention_execution(self, retention_id, exec_id, expect_status_code = 200, **kwargs):
        r, status_code, _ = self._get_client(**kwargs).operate_retention_execution_with_http_info(retention_id, exec_id, {"action":"stop"})
        base._assert_status_code(expect_status_code, status_code)
        return r

    def get_retention_executions(self, retention_id, expect_status_code = 200, **kwargs):
        r, status_code, _ = self._get_client(**kwargs).list_retention_executions_with_http_info(retention_id)
        base._assert_status_code(expect_status_code, status_code)
        return r

    def get_retention_exec_tasks(self, retention_id, exec_id, expect_status_code = 200, **kwargs):
        r, status_code, _ = self._get_client(**kwargs).list_retention_tasks_with_http_info(retention_id, exec_id)
        base._assert_status_code(expect_status_code, status_code)
        return r

    def get_retention_exec_task_log(self, retention_id, exec_id, task_id, expect_status_code = 200, **kwargs):
        r, status_code, _ = self._get_client(**kwargs).get_retention_task_log_with_http_info(retention_id, exec_id, task_id)
        base._assert_status_code(expect_status_code, status_code)
        return r

    def get_retention_metadatas(self, expect_status_code = 200, **kwargs):
        r, status_code, _ = self._get_client(**kwargs).get_rentenition_metadata_with_http_info()
        base._assert_status_code(expect_status_code, status_code)
        return r




import unittest
import semver
import tempfile
import os
import tarfile
import shutil
from pathlib import Path
from migrate_chart import extract_chart_name_and_version
from migrate_chart import read_chart_version  

class TestExtractChartNameAndVersion(unittest.TestCase):
    def test_valid_chart_name(self):
        filepath = Path("my-project/my-chart-v1.0.0.tgz")
        name, version = extract_chart_name_and_version(filepath)
        self.assertEqual(name, "my-chart")
        self.assertEqual(version, "v1.0.0")

    def test_invalid_chart_name(self):
        filepath = Path("my-project/mychart.tgz")
        name, version = extract_chart_name_and_version(filepath)
        self.assertIsNone(name)
        self.assertIsNone(version)

    # def test_pure_digit(self):
    #     filepath = Path("my-project/my-chart-8.0.0-5.tgz")
    #     name, version = extract_chart_name_and_version(filepath)
    #     self.assertEqual(name, "my-chart")
    #     self.assertEqual(version, "8.0.0")

    # def test_digit_startv(self):
    #     filepath = Path("my-project/my-chart-v8.0.0-5.tgz")
    #     name, version = extract_chart_name_and_version(filepath)
    #     self.assertEqual(name, "my-chart")
    #     self.assertEqual(version, "8.0.0")

    def test_parse_version(self):
        temp_dir = tempfile.mkdtemp()
        file_name = "/Users/daojunz/Downloads/cert-manager/sample/cert-manager-8.0.0-5.tgz"
        name, version = read_chart_version(file_name)
        print(name)
        print(version)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
import base

def generate_cert():
    command = ["notation", "cert", "generate-test", "--default", "wabbit-networks.io"]
    base.run_command(command)

def sign_artifact(artifact):
    command = ["notation", "sign", "-d", "--allow-referrers-api", artifact]
    base.run_command(command)



from __future__ import absolute_import
import unittest


from testutils import ADMIN_CLIENT, suppress_urllib3_warning
from testutils import harbor_server
from testutils import admin_user
from testutils import admin_pwd
from testutils import created_project
from testutils import created_user
from testutils import TEARDOWN
from library.repository import push_self_build_image_to_project
from library.repository import Repository


class TestLimitedGuestGetRepository(unittest.TestCase):


    @suppress_urllib3_warning
    def setUp(self):
        self.repository = Repository()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        print("Case completed")

    def testLimitedGuestGetRepository(self):
        """
        Test case:
            Limited Guest GetRepository
        Test step and expected result:
            1. Create a new user(UA)
            2. Create a private project(PA)
            3. Add (UA) as "Limited Guest" to this (PA)
            4. Push an image to project(PA)
            5. Call the "GetRepository" API, it should return 200 status code and project_id should be as expected, and the name should be "ProjectName/ImageName"
            6. Delete repository(RA)
        """
        url = ADMIN_CLIENT["endpoint"]
        user_001_password = "Aa123456"
        # 1. Create a new user(UA)
        with created_user(user_001_password) as (user_id, user_name):
            #2. Create a new private project(PA) by user(UA);
            #3. Add user(UA) as a member of project(PA) with "Limited Guest" role;
            with created_project(metadata={"public": "false"}, user_id=user_id, member_role_id=5) as (project_id, project_name):
                #4. Push an image to project(PA) by user(UA), then check the project quota usage;
                image, tag = "goharbor/alpine", "3.10"
                push_self_build_image_to_project(project_name, harbor_server, admin_user, admin_pwd, image, tag)

                #5. Call the "GetRepository" API, it should return 200 status code and the "name" attribute is "ProjectName/ImageName"
                USER_CLIENT=dict(endpoint=url, username=user_name, password=user_001_password)
                repository_data = self.repository.get_repository(project_name, "goharbor%2Falpine", **USER_CLIENT)
                self.assertEqual(repository_data.project_id, project_id)
                self.assertEqual(repository_data.name, project_name + "/" + image)

                #6. Delete repository(RA)
                self.repository.delete_repository(project_name, "goharbor%2Falpine", **ADMIN_CLIENT)


if __name__ == '__main__':
    unittest.main()




import unittest

from testutils import harbor_server, ADMIN_CLIENT, suppress_urllib3_warning
from library.project import Project
from library.user import User
from library.repository import push_image_to_project
from library.scan import Scan
from library.artifact import Artifact
from library.securityhub import SecurityHub

class TestSecurityHub(unittest.TestCase):


    @suppress_urllib3_warning
    def setUp(self):
        self.project = Project()
        self.user = User()
        self.scan = Scan()
        self.artifact = Artifact()
        self.securityhub = SecurityHub()
        self.image = "ghcr.io/goharbor/notary-server-photon"
        self.new_image = "notary-server-photon"
        self.tag = "v2.2.0"
        self.digest = "sha256:379bf2c7cd55b4214ced7f9a885b46f81992eb01abebfd67693f5cb394611ad1"


    def testSecurityHub(self):
        """
        Test case:
            Security Hub
        Test step and expected result:
            1. Create a new user(UA);
            2. Create a new project(PA) by user(UA);
            3. Push a new image(IA) in project(PA) by user(UA);
            4. Send scan image command and get tag(TA) information to check scan result, it should be finished;
            5. Get vulnerability system summary;
            6. Check the vulnerability system summary is correct;
            7. Check the Get the vulnerability list API;
            8. Check Get Vulnerability List API Search by severity;
            9. Check Get Vulnerability List API Search by project name;
            10. Check Get Vulnerability List API Search by repository name;
            11. Check Get Vulnerability List API Search by tag;
            12. Check Get Vulnerability List API Search by digest;
            13. Check Get Vulnerability List API Search by package;
            14. Check Get Vulnerability List API Search by cve_id;
            15. Check Get Vulnerability List API Search by cvss3;
            16. Check Get Vulnerability List API Search by all options;
        """
        url = ADMIN_CLIENT["endpoint"]
        user_password = "Aa123456"

        # 1. Create user(UA)
        _, user_name = self.user.create_user(user_password=user_password, **ADMIN_CLIENT)
        user_client = dict(endpoint=url, username=user_name, password=user_password, with_scan_overview = True)

        # 2. Create private project(PA) by user(UA)
        project_id, project_name = self.project.create_project(metadata={"public": "false"}, **user_client)

        # 3. Push a new image(IA) in project(PA) by user(UA)
        repository_name, tag = push_image_to_project(project_name, harbor_server, user_name, user_password, self.image, self.tag, new_image=self.new_image)

        # 4. Send scan image command and get tag(TA) information to check scan result, it should be finished
        self.scan.scan_artifact(project_name, self.new_image, tag, **user_client)
        self.artifact.check_image_scan_result(project_name, self.new_image, tag, **user_client)

        # 5. Get vulnerability system summary
        security_summary = self.securityhub.get_security_summary(**ADMIN_CLIENT)

        # 6. Check the vulnerability system summary is correct
        self.check_security_summary(security_summary, repository_name)

        # 7. Check the Get the vulnerability list API
        vulnerabilities = self.securityhub.list_vulnerabilities(**ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities)

        # 8. Check Get Vulnerability List API Search by severity
        vulnerabilities = self.securityhub.list_vulnerabilities(q="severity=Critical", **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, severity="Critical")
        vulnerabilities = self.securityhub.list_vulnerabilities(q="severity=High", **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, severity="High")
        vulnerabilities = self.securityhub.list_vulnerabilities(q="severity=Medium", **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, severity="Medium")
        vulnerabilities = self.securityhub.list_vulnerabilities(q="severity=Low", **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, severity="Low")

        # 9. Check Get Vulnerability List API Search by project name
        vulnerabilities = self.securityhub.list_vulnerabilities(q="project_id=%s" % project_id, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, project_id=project_id)
        vulnerability = vulnerabilities[0]

        # 10. Check Get Vulnerability List API Search by repository name
        vulnerabilities = self.securityhub.list_vulnerabilities(q="repository_name=%s" % repository_name, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, repository_name=repository_name)

        # 11. Check Get Vulnerability List API Search by tag
        vulnerabilities = self.securityhub.list_vulnerabilities(q="tag=%s" % tag, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, tag=tag)

        # 12. Check Get Vulnerability List API Search by digest
        vulnerabilities = self.securityhub.list_vulnerabilities(q="digest=%s" % self.digest, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, digest=self.digest)

        # 13. Check Get Vulnerability List API Search by package
        vulnerabilities = self.securityhub.list_vulnerabilities(q="package=%s" % vulnerability.package, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, package=vulnerability.package)

        # 14. Check Get Vulnerability List API Search by cve_id
        vulnerabilities = self.securityhub.list_vulnerabilities(q="cve_id=%s" % vulnerability.cve_id, **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, cve_id=vulnerability.cve_id)

        # 15. Check Get Vulnerability List API Search by cvss3
        vulnerabilities = self.securityhub.list_vulnerabilities(q="cvss_score_v3=[%s~%s]" % (vulnerability.cvss_v3_score, vulnerability.cvss_v3_score), **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, cvss3_from=vulnerability.cvss_v3_score, cvss3_to=vulnerability.cvss_v3_score)

        # 16. Check Get Vulnerability List API Search by all options
        vulnerabilities = self.securityhub.list_vulnerabilities(q="severity=%s,project_id=%s,repository_name=%s,tag=%s,digest=%s,package=%s,cve_id=%s,cvss_score_v3=[%s~%s]" %
                                                                (vulnerability.severity, vulnerability.project_id, vulnerability.repository_name, vulnerability.tags[0], vulnerability.digest, vulnerability.package, vulnerability.cve_id, vulnerability.cvss_v3_score, vulnerability.cvss_v3_score), **ADMIN_CLIENT)
        self.check_vulnerabilities(vulnerabilities, severity=vulnerability.severity, project_id=vulnerability.project_id, repository_name=vulnerability.repository_name, tag=vulnerability.tags[0], digest=vulnerability.digest, package=vulnerability.package, cve_id=vulnerability.cve_id, cvss3_from=vulnerability.cvss_v3_score, cvss3_to=vulnerability.cvss_v3_score)

    def check_security_summary(self, security_summary, repository_name):
        # Check the summary is correct
        self.assertTrue(security_summary.critical_cnt > 0)
        self.assertTrue(security_summary.high_cnt > 0)
        self.assertTrue(security_summary.medium_cnt > 0)
        self.assertTrue(security_summary.low_cnt > 0)
        self.assertTrue(security_summary.fixable_cnt > 0)
        self.assertTrue(security_summary.scanned_cnt > 0)
        self.assertTrue(security_summary.total_artifact > 0)
        self.assertTrue(security_summary.total_vuls > 0)
        # Check the dangerous artifacts is correct
        dangerous_artifacts = security_summary.dangerous_artifacts
        self.assertTrue(0 < len(dangerous_artifacts) <= 5)
        artifact_sorted_list = sorted(dangerous_artifacts, key=lambda artifact: (artifact.critical_cnt, artifact.high_cnt, artifact.medium_cnt), reverse=True)
        self.assertListEqual(dangerous_artifacts, artifact_sorted_list)
        dangerous_artifacts_dict = {item.repository_name: item for item in dangerous_artifacts}
        self.assertIn(repository_name, dangerous_artifacts_dict.keys())
        for key, value in dangerous_artifacts_dict.items():
            if key == repository_name:
                self.assertEqual(value.digest, self.digest)
                self.assertTrue(value.critical_cnt > 0)
                self.assertTrue(value.high_cnt > 0)
                self.assertTrue(value.medium_cnt > 0)
                break
        # Check the dangerous cves is correct
        dangerous_cves = security_summary.dangerous_cves
        self.assertTrue(0 < len(dangerous_cves) <= 5)
        cve_sorted_list = sorted(dangerous_cves, key=lambda cve: float(cve.cvss_score_v3), reverse=True)
        self.assertListEqual(dangerous_cves, cve_sorted_list)


    def check_vulnerabilities(self, vulnerabilities, severity=None, cve_id=None, cvss3_from=0.0, cvss3_to=10.0, project_id=None, repository_name=None, package=None, tag=None, digest=None):
        for vulnerability in vulnerabilities:
            self.assertEqual(vulnerability.severity, severity) if severity is not None else self.assertIsNotNone(vulnerability.severity)
            self.assertEqual(vulnerability.cve_id, cve_id) if cve_id is not None else self.assertIsNotNone(vulnerability.cve_id)
            self.assertEqual(vulnerability.project_id, project_id) if project_id is not None else self.assertIsNotNone(vulnerability.project_id)
            self.assertEqual(vulnerability.repository_name, repository_name) if repository_name is not None else self.assertIsNotNone(vulnerability.repository_name)
            self.assertEqual(vulnerability.package, package) if package is not None else self.assertIsNotNone(vulnerability.package)
            self.assertEqual(vulnerability.tags[0], tag) if tag is not None else self.assertIsNotNone(vulnerability.tags[0])
            self.assertEqual(vulnerability.digest, digest) if digest is not None else self.assertIsNotNone(vulnerability.digest)
            self.assertTrue(cvss3_from <= vulnerability.cvss_v3_score <= cvss3_to)
            self.assertIsNotNone(vulnerability.desc)
            self.assertIsNotNone(vulnerability.links)
            self.assertIsNotNone(vulnerability.version)


    def testSecurityHubAPIValidate(self):
        """
        Test case:
            Log Rotaion Permission API
        Test step and expected result:
            1. Create a new user(UA);
            2. User(UA) should not have permission to call get vulnerability system summary API;
            3. User(UA) should not have permission to call get Vulnerability List API;
        """
        url = ADMIN_CLIENT["endpoint"]
        user_password = "Aa123456"
        # 1. Create user(UA)
        _, user_name = self.user.create_user(user_password=user_password, **ADMIN_CLIENT)
        user_client = dict(endpoint=url, username=user_name, password=user_password)
        # 2. User(UA) should not have permission to call get vulnerability system summary API
        self.securityhub.get_security_summary(expect_status_code=403, expect_response_body="FORBIDDEN", **user_client)
        # 3. User(UA) should not have permission to call get Vulnerability List API
        self.securityhub.list_vulnerabilities(expect_status_code=403, expect_response_body="FORBIDDEN", **user_client)


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import

import unittest

from testutils import harbor_server, suppress_urllib3_warning
from testutils import TEARDOWN
from testutils import ADMIN_CLIENT
from library.project import Project
from library.user import User
from library.repository import push_self_build_image_to_project
from library.repository import Repository

class TestProjects(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.project = Project()
        self.user = User()
        self.repo = Repository()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        #1. Delete repository(RA) by admin;
        self.repo.delete_repository(TestProjects.project_alice_name, TestProjects.repo_name.split('/')[1], **ADMIN_CLIENT)

        #2. Delete project(Alice);
        self.project.delete_project(TestProjects.project_alice_id, **ADMIN_CLIENT)

        #3. Delete user Alice, Bob and Carol.
        self.user.delete_user(TestProjects.user_alice_id, **ADMIN_CLIENT)
        self.user.delete_user(TestProjects.user_bob_id, **ADMIN_CLIENT)
        self.user.delete_user(TestProjects.user_carol_id, **ADMIN_CLIENT)

    def testManageProjectMember(self):
        """
        Test case:
            Manage Project members
        Test step and expected result:
            1. Create user Alice, Bob, Carol;
            2. Create private project(Alice) by Alice, Add a repository to project(Alice) by Alice;
            3. Bob is not a member of project(Alice);
            4. Alice Add Bob as a guest member of project(Alice), Check Bob is a guest member of project(Alice);
            5. Update role of Bob to developer of project(Alice), Check Bob is developer member of project(Alice);
            6. Update role of Bob to admin member of project(Alice), Check Bob is admin member of project(Alice);
            7. Bob add Carol to project(Alice) as a guest member, Carol is a member of project(Alice) as a guest;
            8. Alice delete Bob from project(Alice),
               Bob is no longer a member of project(Alice) and Bob can see project(Alice),
               Carol is still a member of project(Alice) as a guest.
        Tear down:
            1. Delete repository(RA) by admin;
            2. Delete project(Alice);
            3. Delete user Alice, Bob and Carol.
        """
        url = ADMIN_CLIENT["endpoint"]
        user_alice_password = "Aa123456"
        user_bob_password = "Test1@34"
        user_carol_password = "Test1@34"

        #1.1 Create user Alice
        TestProjects.user_alice_id, user_alice_name = self.user.create_user(user_password = user_alice_password, **ADMIN_CLIENT)
        USER_ALICE_CLIENT=dict(endpoint = url, username = user_alice_name, password = user_alice_password)

        #1.2 Create user Bob
        TestProjects.user_bob_id, user_bob_name = self.user.create_user(user_password = user_bob_password, **ADMIN_CLIENT)
        USER_BOB_CLIENT=dict(endpoint = url, username = user_bob_name, password = user_bob_password)

        #1.3 Create user Carol
        TestProjects.user_carol_id, user_carol_name = self.user.create_user(user_password = user_carol_password, **ADMIN_CLIENT)

        #2.1 Create private project(PA) by Alice
        TestProjects.project_alice_id, TestProjects.project_alice_name = self.project.create_project(metadata = {"public": "false"}, **USER_ALICE_CLIENT)

        #2.2 Add a repository to project(PA) by Alice
        TestProjects.repo_name, _ = push_self_build_image_to_project(TestProjects.project_alice_name, harbor_server, user_alice_name, user_alice_password, "test_member", "latest")

        #3. Bob is not a member of project(PA);
        self.project.check_project_member_not_exist(TestProjects.project_alice_id, user_bob_name, **USER_ALICE_CLIENT)

        #4.1 Alice Add Bob as a guest member of project(PA)
        member_id_bob = self.project.add_project_members(TestProjects.project_alice_id, user_id=TestProjects.user_bob_id, member_role_id = 3, **USER_ALICE_CLIENT)

        #4.2 Check Bob is a guest member of project(PA)
        self.project.check_project_members_exist(TestProjects.project_alice_id, user_bob_name, expected_member_role_id = 3, user_name = user_bob_name, user_password = user_bob_password, **USER_ALICE_CLIENT)

        #5.1 Update role of Bob to developer of project(PA)
        self.project.update_project_member_role(TestProjects.project_alice_id, member_id_bob, 2, **USER_ALICE_CLIENT)

        #5.2 Check Bob is developer member of project(PA)
        self.project.check_project_members_exist(TestProjects.project_alice_id, user_bob_name, expected_member_role_id = 2, user_name = user_bob_name, user_password = user_bob_password, **USER_ALICE_CLIENT)

        #6.1 Update role of Bob to admin member of project(PA)
        self.project.update_project_member_role(TestProjects.project_alice_id, member_id_bob, 1, **USER_ALICE_CLIENT)

        #6.2 Check Bob is admin member of project(PA)
        self.project.check_project_members_exist(TestProjects.project_alice_id, user_bob_name, expected_member_role_id = 1, user_name = user_bob_name, user_password = user_bob_password, **USER_ALICE_CLIENT)

        #7.1 Bob add Carol to project(PA) as a guest member.
        self.project.add_project_members(TestProjects.project_alice_id, TestProjects.user_carol_id, member_role_id = 3, **USER_BOB_CLIENT)

        #7.2 Carol is a member of project(PA) as a guest.
        self.project.check_project_members_exist(TestProjects.project_alice_id, user_carol_name, expected_member_role_id = 3, user_name = user_carol_name, user_password = user_carol_password, **USER_ALICE_CLIENT)

        #8.1 Alice delete Bob from project(PA).
        self.project.delete_project_member(TestProjects.project_alice_id, member_id_bob, **USER_ALICE_CLIENT)

        #8.2 Bob is no longer a member of project(PA) and Bob can see project(PA).
        self.project.check_project_member_not_exist(TestProjects.project_alice_id, user_bob_name, **USER_ALICE_CLIENT)

        #8.3 Carol is still a member of project(PA) as a guest.
        self.project.check_project_members_exist(TestProjects.project_alice_id, user_carol_name, expected_member_role_id = 3, user_name = user_carol_name, user_password = user_carol_password, **USER_ALICE_CLIENT)

if __name__ == '__main__':
    unittest.main()




from commands.prepare import prepare
from commands.gencerts import gencert
from commands.migrate import migrate
import click

@click.group()
def cli():
    pass

cli.add_command(prepare)
cli.add_command(gencert)
cli.add_command(migrate)

if __name__ == '__main__':
    cli()



#!/usr/local/bin/python3

import subprocess
import signal
import sys
import os
from pathlib import Path
import tarfile
import yaml

import click
import requests

MIGRATE_CHART_SCRIPT = '/migrate_chart.sh'
HELM_CMD = '/linux-amd64/helm'
CA_UPDATE_CMD = 'update-ca-certificates'
CHART_URL_PATTERN = "https://{host}/api/v2.0/projects/{project}/repositories/{name}/artifacts/{version}"
CHART_SOURCE_DIR = Path('/chart_storage')

errs = []

def print_exist_errs():
    if errs:
        click.echo("There are {} errors exist".format(len(errs)), err=True)
        for e in errs:
            click.echo(e, err=True)
            # Write the error to file
            with open("/chart_storage/migration_errors.txt", "a") as f:
                f.write(e + "\n")

def graceful_exit(signum, frame):
    print_exist_errs()
    sys.exit()

signal.signal(signal.SIGINT, graceful_exit)
signal.signal(signal.SIGTERM, graceful_exit)

def find_chart_yaml(tar, path=''):
    # Iterate through the members of the tarfile
    for member in tar.getmembers():
        # If the member is a directory, recursively search within it
        if member.isdir():
            find_chart_yaml(tar, os.path.join(path, member.name))
        # If the member is a file and its name is 'chart.yaml', return its path
        if "Chart.yaml" in member.name:
            return os.path.join(path, member.name)

def read_chart_version(chart_tgz_path):
    # Open the chart tgz file
    with tarfile.open(chart_tgz_path, 'r:gz') as tar:
        # Find the path to chart.yaml within the tarball
        chart_yaml_path = find_chart_yaml(tar)
        if chart_yaml_path:
            # Extract the chart.yaml file
            chart_yaml_file = tar.extractfile(chart_yaml_path)
            if chart_yaml_file is not None:
                # Load the YAML content from chart.yaml
                chart_data = yaml.safe_load(chart_yaml_file)
                # Read the version from chart.yaml
                version = chart_data.get('version')
                name = chart_data.get('name')
                return name, version
            else:
                raise Exception("Failed to read chart.yaml from the chart tgz file. filename {}".format(chart_tgz_path))
        else:
            raise Exception("chart.yaml not found in the chart tgz file. filename {}".format(chart_tgz_path))

class ChartV2:

    def __init__(self, filepath:Path):
        self.filepath = filepath
        self.project = self.filepath.parts[-2]
        self.name = ""
        self.version = ""
        try:
            self.name, self.version = read_chart_version(filepath)
            if self.name == "" or self.version == "" or self.name is None or self.version is None :
                raise Exception('chart name: {} is illegal'.format('-'.join(parts)))
        except Exception as e:
            click.echo("Skipped chart: {} due to illegal chart name. Error: {}".format(filepath, e), err=True)
        return

    def __check_exist(self, hostname, username, password):
        return requests.get(CHART_URL_PATTERN.format(
                host=hostname,
                project=self.project,
                name=self.name,
                version=self.version),
                auth=requests.auth.HTTPBasicAuth(username, password))

    def migrate(self, hostname, username, password):
        res = self.__check_exist(hostname, username, password)
        if res.status_code == 200:
            raise Exception("Artifact already exist in harbor")
        if res.status_code == 401:
            raise Exception(res.reason)

        oci_ref = "oci://{host}/{project}".format(
            host=hostname,
            project=self.project)

        return subprocess.run([MIGRATE_CHART_SCRIPT, HELM_CMD, self.filepath, oci_ref],
        text=True, stdout=subprocess.DEVNULL, stderr=subprocess.PIPE)


@click.command()
@click.option('--hostname', default='127.0.0.1', help='the password to login harbor')
@click.option('--username', default='admin', help='The username to login harbor')
@click.option('--password', default='Harbor12345', help='the password to login harbor')
def migrate(hostname, username, password):
    """
    Migrate chart v2 to harbor oci registry
    """
    if username != 'admin':
        raise Exception('This operation only allowed for admin')
    subprocess.run([CA_UPDATE_CMD])
    subprocess.run([HELM_CMD, 'registry', 'login', hostname, '--username', username, '--password', password])
    charts = [ChartV2(c) for p in CHART_SOURCE_DIR.iterdir() if p.is_dir() for c in p.iterdir() if c.is_file() and c.name.endswith(".tgz")]
    with click.progressbar(charts, label="Migrating chart ...", length=len(charts),
    item_show_func=lambda x: "{}/{}:{} total errors: {}".format(x.project, x.name, x.version, len(errs)) if x else '') as bar:
        for chart in bar:
            try:
                if chart.name == "" or chart.version == "" :
                    print("skip the chart {} has no name or version info".format(chart.filepath))
                    continue
                result = chart.migrate(hostname, username, password)
                if result.stderr:
                    errs.append("chart: {name}:{version} in {project} has err: {err}".format(
                        name=chart.name,
                        version=chart.version,
                        project=chart.project,
                        err=result.stderr
                    ))
            except Exception as e:
                errs.append("chart: {name}:{version} in {project}, path {path} has err: {err}".format(
                    name=chart.name,
                    version=chart.version,
                    project=chart.project,
                    path = chart.filepath,
                    err=e))
    click.echo("Migration is Done.")
    print_exist_errs()

if __name__ == '__main__':
    migrate()



from __future__ import absolute_import
import unittest

from testutils import harbor_server, suppress_urllib3_warning
from testutils import TEARDOWN
from testutils import ADMIN_CLIENT
from testutils import created_user, created_project
from library.project import Project
from library.user import User
from library.repository import Repository
from library.artifact import Artifact
from library.configurations import Configurations
from library.projectV2 import ProjectV2
from library.repository import push_self_build_image_to_project


class TestAssignRoleToLdapGroup(unittest.TestCase):
    @suppress_urllib3_warning
    def setUp(self):
        self.conf= Configurations()
        self.project = Project()
        self.artifact = Artifact()
        self.repo = Repository()
        self.user= User()

    @unittest.skipIf(TEARDOWN == False, "Test data won't be erased.")
    def tearDown(self):
        print("Case completed")

    def testAssignRoleToLdapGroup(self):
        """
        Test case:
            Assign Role To Ldap Group
        Test step and expected result:
            1. Set LDAP Auth configurations;
            2. Create a new public project(PA) by Admin;
            3. Add 3 member groups to project(PA);
            4. Push image by each member role;
            5. Verfify that admin_user can add project member, dev_user and guest_user can not add project member;
            6. Verfify that admin_user and dev_user can push image, guest_user can not push image;
            7. Verfify that admin_user, dev_user and guest_user can view logs, test user can not view logs.
            8. Delete repository(RA) by user(UA);
            9. Delete project(PA);
        """
        url = ADMIN_CLIENT["endpoint"]
        USER_ADMIN=dict(endpoint = url, username = "admin_user", password = "zhu88jie", repo = "haproxy")
        USER_DEV=dict(endpoint = url, username = "dev_user", password = "zhu88jie", repo = "alpine")
        USER_GUEST=dict(endpoint = url, username = "guest_user", password = "zhu88jie", repo = "busybox")
        USER_TEST=dict(endpoint = url, username = "test", password = "123456")
        USER_MIKE=dict(endpoint = url, username = "mike", password = "zhu88jie")
        #USER001 is in group harbor_group3
        self.conf.set_configurations_of_ldap(ldap_filter="", ldap_group_attribute_name="cn", ldap_group_base_dn="ou=groups,dc=example,dc=com",
                                             ldap_group_search_filter="objectclass=groupOfNames", ldap_group_search_scope=2, **ADMIN_CLIENT)

        with created_project(metadata={"public": "false"}) as (project_id, project_name):
            self.project.add_project_members(project_id, member_role_id = 1, _ldap_group_dn = "cn=harbor_admin,ou=groups,dc=example,dc=com", **ADMIN_CLIENT)
            self.project.add_project_members(project_id, member_role_id = 2, _ldap_group_dn = "cn=harbor_dev,ou=groups,dc=example,dc=com", **ADMIN_CLIENT)
            self.project.add_project_members(project_id, member_role_id = 3, _ldap_group_dn = "cn=harbor_guest,ou=groups,dc=example,dc=com", **ADMIN_CLIENT)

            projects = self.project.get_projects(dict(name=project_name), **USER_ADMIN)
            self.assertTrue(len(projects) == 1) 
            self.assertEqual(1, projects[0].current_user_role_id)

            #Mike has logged in harbor in previous test.
            mike = self.user.get_user_by_name(USER_MIKE["username"], **ADMIN_CLIENT)

            #Verify role difference in add project member feature, to distinguish between admin and dev role
            self.project.add_project_members(project_id, user_id=mike.user_id, member_role_id = 3, **USER_ADMIN)
            self.project.add_project_members(project_id, user_id=mike.user_id, member_role_id = 3, expect_status_code=403, **USER_DEV)
            self.project.add_project_members(project_id, user_id=mike.user_id, member_role_id = 3, expect_status_code=403, **USER_GUEST)

            repo_name_admin, _  = push_self_build_image_to_project(project_name, harbor_server, USER_ADMIN["username"], USER_ADMIN["password"], USER_ADMIN["repo"], "latest")
            artifacts = self.artifact.list_artifacts(project_name, USER_ADMIN["repo"], **USER_ADMIN)
            self.assertTrue(len(artifacts) == 1)
            repo_name_dev, _ = push_self_build_image_to_project(project_name, harbor_server, USER_DEV["username"], USER_DEV["password"], USER_DEV["repo"], "latest")
            artifacts = self.artifact.list_artifacts(project_name, USER_DEV["repo"], **USER_DEV)
            self.assertTrue(len(artifacts) == 1)
            push_self_build_image_to_project(project_name, harbor_server, USER_GUEST["username"], USER_GUEST["password"], USER_GUEST["repo"], "latest", expected_error_message = "unauthorized to access repository")
            artifacts = self.artifact.list_artifacts(project_name, USER_GUEST["repo"], **USER_GUEST)
            self.assertTrue(len(artifacts) == 0)

            self.assertTrue(self.project.query_user_logs(project_name, **USER_ADMIN)>0, "admin user can see logs")
            self.assertTrue(self.project.query_user_logs(project_name, status_code=403, **USER_DEV)==0, "dev user can not see any logs")
            self.assertTrue(self.project.query_user_logs(project_name, status_code=403, **USER_GUEST)==0, "guest user can not see any logs")
            self.assertTrue(self.project.query_user_logs(project_name, status_code=403, **USER_TEST)==0, "test user can not see any logs")

            self.repo.delete_repository(project_name, repo_name_admin.split('/')[1], **USER_ADMIN)
            self.repo.delete_repository(project_name, repo_name_dev.split('/')[1], **USER_ADMIN)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import base
import json
import docker_api
from testutils import DOCKER_USER, DOCKER_PWD

def load_bundle(service_image, invocation_image):
    bundle_file = "./tests/apitests/python/bundle_data/bundle.json"
    bundle_tmpl_file = "./tests/apitests/python/bundle_data/bundle.json.tmpl"
    with open(bundle_tmpl_file,'r') as load_f:
        load_dict = json.load(load_f)
        load_dict["images"]["hello"]["image"] = service_image
        load_dict["invocationImages"][0]["image"] = invocation_image
        bundle_str = json.dumps(load_dict)
        with open(bundle_file,'w') as dump_f:
            dump_f.write(bundle_str)
            dump_f.close()
        return bundle_file

def cnab_fixup_bundle(bundle_file, target, auto_update_bundle = True):
    fixed_bundle_file = "./tests/apitests/python/bundle_data/fixed-bundle.json"
    command = ["cnab-to-oci", "--log-level", "debug", "fixup", bundle_file, "--target", target, "--bundle", fixed_bundle_file]
    if auto_update_bundle == True:
         command.append("--auto-update-bundle")
    ret = base.run_command(command)
    return fixed_bundle_file

def cnab_push_bundle(bundle_file, target):
    command = ["cnab-to-oci", "push", bundle_file, "--target", target, "--auto-update-bundle"]
    ret = base.run_command(command)
    for line in ret.split("\n"):
        line = line.replace('\"', '')
        if line.find('sha256') >= 0:
            return line[-71:]
    raise Exception(r"Fail to get sha256 in returned data: {}".format(ret))

def push_cnab_bundle(harbor_server, user, password, service_image, invocation_image, target, auto_update_bundle = True):
    docker_api.docker_info_display()

    docker_api.docker_login_cmd(harbor_server, user, password, enable_manifest = False)
    bundle_file = load_bundle(service_image, invocation_image)
    fixed_bundle_file = cnab_fixup_bundle(bundle_file, target, auto_update_bundle = auto_update_bundle)
    sha256 = cnab_push_bundle(fixed_bundle_file, target)
    return sha256



# -*- coding: utf-8 -*-

import base


def helm_registry_login(ip, user, password):
    command = ["helm", "registry", "login", ip, "-u", user, "-p", password, "--insecure"]
    base.run_command(command)

def helm_package(file_path):
    command = ["helm", "package", file_path]
    base.run_command(command)

def helm_push(file_path, ip, project_name):
    command = ["helm", "push", file_path, "oci://{}/{}".format(ip, project_name), "--insecure-skip-tls-verify"]
    base.run_command(command)



# -*- coding: utf-8 -*-

import v2_swagger_client
from library.base import _assert_status_code
from v2_swagger_client.models.role_request import RoleRequest
from v2_swagger_client.rest import ApiException

import base


def is_member_exist_in_project(members, member_user_name, expected_member_role_id = None):
    result = False
    for member in members:
        if member.entity_name == member_user_name:
            if expected_member_role_id != None:
                if member.role_id == expected_member_role_id:
                    return True
            else:
                return True
    return result

def get_member_id_by_name(members, member_user_name):
    for member in members:
        if member.entity_name == member_user_name:
            return member.id
    return None

class Project(base.Base):
    def __init__(self, username=None, password=None):
        kwargs = dict(api_type="projectv2")
        if username and password:
            kwargs["credential"] = base.Credential('basic_auth', username, password)
        super(Project, self).__init__(**kwargs)

    def create_project(self, name=None, registry_id=None, metadata=None, expect_status_code = 201, expect_response_body = None, **kwargs):
        if name is None:
            name = base._random_name("project")
        if metadata is None:
            metadata = {}
        if registry_id is None:
            registry_id = registry_id
        project = v2_swagger_client.ProjectReq(project_name=name, registry_id = registry_id, metadata=metadata)
        try:
            _, status_code, header = self._get_client(**kwargs).create_project_with_http_info(project)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return
        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(201, status_code)
        return base._get_id_from_header(header), name

    def get_projects(self, params, **kwargs):
        data = []
        data, status_code, _ = self._get_client(**kwargs).list_projects_with_http_info(**params)
        base._assert_status_code(200, status_code)
        return data

    def get_project_id(self, project_name, **kwargs):
        project_data = self.get_projects(dict(), **kwargs)
        actual_count = len(project_data)
        if actual_count == 1 and str(project_data[0].project_name) != str(project_name):
            return project_data[0].project_id
        else:
            return None

    def projects_should_exist(self, params, expected_count = None, expected_project_id = None, **kwargs):
        project_data = self.get_projects(params, **kwargs)
        actual_count = len(project_data)
        if expected_count is not None and actual_count!= expected_count:
            raise Exception(r"Private project count should be {}.".format(expected_count))
        if expected_project_id is not None and actual_count == 1 and str(project_data[0].project_id) != str(expected_project_id):
            raise Exception(r"Project-id check failed, expect {} but got {}, please check this test case.".format(str(expected_project_id), str(project_data[0].project_id)))

    def check_project_name_exist(self, name=None, **kwargs):
        try:
            _, status_code, _ = self._get_client(**kwargs).head_project_with_http_info(name)
        except ApiException as e:
            status_code = -1
        return {
            200: True,
            404: False,
        }.get(status_code,False)

    def get_project(self, project_id, expect_status_code = 200, expect_response_body = None, **kwargs):
        try:
            data, status_code, _ = self._get_client(**kwargs).get_project_with_http_info(project_id)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return

        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(200, status_code)
        print("Project {} info: {}".format(project_id, data))
        return data

    def update_project(self, project_id, expect_status_code=200, metadata=None, cve_allowlist=None, **kwargs):
        project = v2_swagger_client.ProjectReq(metadata=metadata, cve_allowlist=cve_allowlist)
        try:
            _, sc, _ = self._get_client(**kwargs).update_project_with_http_info(project_id, project)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
        else:
            base._assert_status_code(expect_status_code, sc)

    def delete_project(self, project_id, expect_status_code = 200, **kwargs):
        _, status_code, _ = self._get_client(**kwargs).delete_project_with_http_info(project_id)
        base._assert_status_code(expect_status_code, status_code)

    def get_project_log(self, project_name, expect_status_code = 200, **kwargs):
        body, status_code, _ = self._get_client(**kwargs).get_log_exts_with_http_info(project_name)
        base._assert_status_code(expect_status_code, status_code)
        return body

    def filter_project_logs(self, project_name, operator, resource, resource_type, operation, **kwargs):
        access_logs = self.get_project_log(project_name, **kwargs)
        count = 0
        for each_access_log in list(access_logs):
            if each_access_log.username == operator and \
                    each_access_log.resource_type == resource_type and \
                    each_access_log.resource == resource and \
                    each_access_log.operation == operation:
                count = count + 1
        return count

    def get_project_members(self, project_id, **kwargs):
        kwargs['api_type'] = 'member'
        return self._get_client(**kwargs).list_project_members(project_id)

    def get_project_member(self, project_id, member_id, expect_status_code = 200, expect_response_body = None, **kwargs):
        kwargs['api_type'] = 'member'
        data = []
        try:
            data, status_code, _ = self._get_client(**kwargs).get_project_member_with_http_info(project_id, member_id,)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
            if expect_response_body is not None:
                base._assert_status_body(expect_response_body, e.body)
            return

        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(200, status_code)
        return data

    def get_project_member_id(self, project_id, member_user_name, **kwargs):
        kwargs['api_type'] = 'member'
        members = self.get_project_members(project_id, **kwargs)
        result = get_member_id_by_name(list(members), member_user_name)
        if result == None:
            raise Exception(r"Failed to get member id of member {} in project {}.".format(member_user_name, project_id))
        else:
            return result

    def check_project_member_not_exist(self, project_id, member_user_name, **kwargs):
        kwargs['api_type'] = 'member'
        members = self.get_project_members(project_id, **kwargs)
        result = is_member_exist_in_project(list(members), member_user_name)
        if result == True:
            raise Exception(r"User {} should not be a member of project with ID {}.".format(member_user_name, project_id))

    def check_project_members_exist(self, project_id, member_user_name, expected_member_role_id = None, **kwargs):
        kwargs['api_type'] = 'member'
        members = self.get_project_members(project_id, **kwargs)
        result = is_member_exist_in_project(members, member_user_name, expected_member_role_id = expected_member_role_id)
        if result == False:
            raise Exception(r"User {} should be a member of project with ID {}.".format(member_user_name, project_id))

    def update_project_member_role(self, project_id, member_id, member_role_id, expect_status_code = 200, **kwargs):
        kwargs['api_type'] = 'member'
        role = RoleRequest(role_id = member_role_id)
        data, status_code, _ = self._get_client(**kwargs).update_project_member_with_http_info(project_id, member_id, role = role)
        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(200, status_code)
        return data

    def delete_project_member(self, project_id, member_id, expect_status_code = 200, **kwargs):
        kwargs['api_type'] = 'member'
        _, status_code, _ = self._get_client(**kwargs).delete_project_member_with_http_info(project_id, member_id)
        base._assert_status_code(expect_status_code, status_code)
        base._assert_status_code(200, status_code)

    def add_project_members(self, project_id, user_id = None, member_role_id = None, _ldap_group_dn=None, expect_status_code = 201, **kwargs):
        kwargs['api_type'] = 'member'
        projectMember = v2_swagger_client.ProjectMember()
        if user_id is not None:
           projectMember.member_user = {"user_id": int(user_id)}
        if member_role_id is None:
            projectMember.role_id = 1
        else:
            projectMember.role_id = member_role_id
        if _ldap_group_dn is not None:
            projectMember.member_group = v2_swagger_client.UserGroup(ldap_group_dn=_ldap_group_dn)

        data = []
        try:
            data, status_code, header = self._get_client(**kwargs).create_project_member_with_http_info(project_id, project_member = projectMember)
        except ApiException as e:
            base._assert_status_code(expect_status_code, e.status)
        else:
            base._assert_status_code(expect_status_code, status_code)
            return base._get_id_from_header(header)

    def query_user_logs(self, project_name, status_code=200, **kwargs):
        try:
            logs = self.get_project_log(project_name, expect_status_code=status_code, **kwargs)
            count = 0
            for log in list(logs):
                count = count + 1
            return count
        except ApiException as e:
            _assert_status_code(status_code, e.status)
            return 0



# -*- coding: utf-8 -*-

import time
import base
import v2_swagger_client
from v2_swagger_client.rest import ApiException

class ProjectV2(base.Base, object):
    def __init__(self):
        super(ProjectV2,self).__init__(api_type = "projectv2")

    def get_project_log(self, project_name, expect_status_code = 200, **kwargs):
        body, status_code, _ = self._get_client(**kwargs).get_log_exts_with_http_info(project_name)
        base._assert_status_code(expect_status_code, status_code)
        return body

    def filter_project_logs(self, project_name, operator, resource, resource_type, operation, **kwargs):
        access_logs = self.get_project_log(project_name, **kwargs)
        count = 0
        for each_access_log in list(access_logs):
            if each_access_log.username == operator and \
                    each_access_log.resource_type == resource_type and \
                    each_access_log.resource == resource and \
                    each_access_log.operation == operation:
                count = count + 1
        return count

